# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# lib_backend.py
#  
# classes de base pour la gestion des serveurs sous zephir
#       
###########################################################################
"""fonctions utiles pour l'accès à la base de données (via adbapi)
"""
from zephir.backend import config
try:
    from migration_perso import migration_perso, destinations_perso
except:
    migration_perso = {}
    destinations_perso = []
from zephir.utils.creolewrap import ZephirDict
from zephir.backend.uucp_utils import UUCP
from twisted.enterprise import adbapi
from twisted.python import log
from zephir.monitor.agentmanager.util import md5files
from twisted.internet.utils import getProcessOutputAndValue
import psycopg2 as PgSQL
import os, sys, time, string, traceback, apt_pkg, re
import base64
from ConfigParser import ConfigParser
from glob import glob
import xmlrpclib, os

class ResourceAuthError(Exception):
    pass

class CxPool(dict):
    """dictionnaire des connexions postgresql en cours"""
    
    def create(self):
        cx = PgSQL.connect(database=config.DB_NAME,user=config.DB_USER,password=config.DB_PASSWD)
        cu = cx.cursor()
        self[cu] = cx
        return cu

    def commit(self,cu):
        cu.close()
        self[cu].commit()
        self[cu].close()
        del(self[cu])

    def rollback(self,cu):
        cu.close()
        self[cu].rollback()
        self[cu].close()
        del(self[cu])

    def close(self,cu):
        cu.close()
        self[cu].close()
        del(self[cu])

class AptChecker:
    """gestionnaire de vérification d'état des paquets d'un serveur
    """
    def __init__(self):
        self.pkg_file = {}
        self.packages = {}
        self.timestamp = {}
        for zephir_version, data in config.DISTRIBS.items():
            if zephir_version != 1:
                codename, version = data
                self.pkg_file[zephir_version] = os.path.join(os.path.abspath(config.PATH_ZEPHIR),'packages_%s.ini' % codename)
                self.packages[zephir_version] = ConfigParser()
                if not os.path.isfile(self.pkg_file[zephir_version]):
                    f = file(self.pkg_file[zephir_version],'w')
                    f.close()
                    # la liste des paquets n'a jamais été vérifiée, on lance la procédure de recherche
                    print "** récupération de la liste des paquets disponibles en mise à jour"
                    os.system('/usr/share/eole/check_maj_eole.py')
                self.timestamp[zephir_version] = os.path.getmtime(self.pkg_file[zephir_version])
                self.packages[zephir_version].read(self.pkg_file[zephir_version])
        # création d'une regexp compilée pour les paquets à ignorer
        if len(config.holded_packages) > 0:
            self.holded_pattern = re.compile("|".join(config.holded_packages))
        else:
            self.holded_pattern = None

    def check_packages(self, pkg_liste, version, type_maj = 'complete', show_installed = False):
        """vérifie l'état de mise à jour des paquets
        type_maj : type de mise à jour (minimale/complete)
        show_installed : si True, renvoie la liste de tous les paquets installés
                         par défaut, seulement les paquets non à jour
        """
        apt_pkg.InitSystem()
        if self.timestamp[version] != os.path.getmtime(self.pkg_file[version]):
            # la liste des paquets dispos a changé
            self.packages[version].read(self.pkg_file[version])
            self.timestamp[version] = os.path.getmtime(self.pkg_file[version])
        outdated = []
        for pkg_name, pkg_version in pkg_liste:
            if (not self.holded_pattern.match(pkg_name)) or (self.holded_pattern is None):
                try:
                    # les paquets n'ayant jamais eu de mise à jour ne sont pas présents
                    if self.packages[version].has_section(pkg_name):
                        maj_version = self.packages[version].get(pkg_name,type_maj)
                        if (apt_pkg.VersionCompare(maj_version, pkg_version) > 0) or (show_installed == True):
                            # on renvoie, le paquet, la version installée, et la version sur le serveur de maj
                            outdated.append((pkg_name, pkg_version, maj_version))
                            continue
                except:
                    traceback.print_exc()
                # listing des paquets installés, on renvoie le paquet
                # même si il n'est pas dans les repository de maj
                if show_installed == True:
                    outdated.append((pkg_name, pkg_version, pkg_version))
        return outdated

cx_pool = CxPool()
maj_checker = AptChecker()

class Serveur:
    """classe utilitaire pour récupérer diverses données sur un serveur"""

    def __init__(self, id_s, cu=None, data=None):
        """initialise l'objet serveur et récupère les infos
        """
        self.version = 'creole2'
        # récupération des serveurs
        self.id_s = int(id_s)
        # date de dernière modification du serveur
        self.modified=time.time()
        # date de création de l'objet
        self.created=time.time()
        cursor = cu
        if cu == None:
            cursor = cx_pool.create()
        try:
            if data == None:
                # le rne ne peut pas être modifié : on le stocke une fois pour toutes
                cursor.execute("select rne from serveurs where id=%s" % str(self.id_s))
                data = cursor.fetchone()
                self.rne = data[0]
                # appel de la fonction update_data pour les données modifiables
                self.update_data(cursor)
            else:
                self.rne = data[0]
            self.confdir = os.path.join(config.PATH_ZEPHIR,'conf',self.rne, str(self.id_s))
            if cu == None:
                cx_pool.close(cursor)
        except:
            traceback.print_exc()
            if cu == None:
                cx_pool.close(cursor)

    def update_data(self, cu=None, data=None):
        """recharge les données des serveurs
        """
        # récupération de l'établissement et de l'état
        cursor = cu
        if cu == None:
            cursor = cx_pool.create()
        try:
            if data == None:
                cursor.execute("select libelle, module_actuel, variante, timeout, etat, md5s, maj from serveurs where id=%s" % str(self.id_s))
                data = cursor.fetchone()
            # données du serveur
            self.libelle = data[0]
            self.id_mod = int(data[1])
            self.id_var = int(data[2])
            try:
                self.md5s = int(data[5])
            except:
                self.md5s = -1
            try:
                self.maj = int(data[6])
            except:
                self.maj = -1
            try:
                self.timeout = int(data[3])
            except:
                self.timeout = 0
            if data[4] == None:
                self.status = -1
            else:
                self.status = int(data[4])
            # libelle etablissement
            cursor.execute("select libelle from etablissements where rne='%s'" % self.rne)
            data = cursor.fetchone()
            self.etab = data[0]
            # libelle module
            cursor.execute("select libelle,version from modules where id='%s'" % self.id_mod)
            data = cursor.fetchone()
            self.module = data[0]
            try:
                self.module_version = int(data[1])
            except:
                self.module_version = 3
            # détection des modules utilisant creole1
            OLD_MODULES =  config.liste_modules[1]
            if self.module in OLD_MODULES:
                self.version = 'creole1'
            else:
                self.version = 'creole2'
            #libelle variante
            cursor.execute("select libelle from variantes where id='%s'" % self.id_var)
            data = cursor.fetchone()
            self.variante = data[0]
            self.modified = time.time()
            if cu == None:
                cx_pool.close(cursor)
        except:
            traceback.print_exc()
            if cu == None:
                cx_pool.close(cursor)

    def __repr__(self):
        """affichage du serveur
        """
        return """%s => %s (%s)""" % (self.id_s, self.module, self.rne)

    def __str__(self):
        """affichage du serveur
        """
        return """%s => %s (%s) - %s (%s)""" % (self.id_s, self.libelle, self.module, self.etab, self.rne)

    #####################
    ## LECTURE DE DONNEES

    def get_libelle(self):
        return self.libelle

    def get_rne(self):
        return self.rne

    def get_etab(self):
        return self.etab

    def get_module(self):
        return self.module

    def get_variante(self):
        return self.variante

    def get_timeout(self):
        return self.timeout

    def get_confdir(self):
        return self.confdir

    def get_config(self, mode="modif_config", encode=False):
        """retourne la configuration eole du serveur (zephir.eol)
        """
        # récupération du dictionnaire et des dicos locaux
        if self.version == 'creole1':
            # on recherche la liste des dictionnaires à charger
            creole_files = [self.confdir+os.sep+'dictionnaire']
            # dictionnaire de la variante
            creole_files.extend(glob(self.confdir+os.sep+'dicos/variante/*.eol'))
            # dictionnaire locaux
            creole_files.extend(glob(self.confdir+os.sep+'dicos/*.eol'))
            dicos = []
            for dic in creole_files:
                try:
                    # lecture du contenu des dictionnaires
                    fic = open(dic,'r')
                    lines = fic.readlines()
                    fic.close()
                    if encode == True:
                        data = [ unicode(line, 'ISO-8859-1').encode('UTF-8') for line in lines ]
                    else:
                        data = lines
                    # stockage du contenu du dictionnaire
                    dicos.append(data)
                except OSError:
                    pass

        if self.version == 'creole2':
            # chargement de configuration pour un serveur Eole2
            dicos = []
            for rep in ['module','local','variante']:
                dicos.append(os.path.join(self.confdir,'dicos',rep))

        dico = ZephirDict(dicos, self.confdir, mode, self.version)
        return dico

    def save_config(self, dico_zeph, mode='config', encode=False, force=False):
        """sauvegarde la configuration eole du serveur (zephir.eol)
        """
        try:
            if mode in ['config','modif_config']:
                fic_zephir = os.path.join(self.confdir,'zephir.eol')
                key = 'config_ok'
            elif mode in ['dico','modif_dico']:
                # si gen_dico, sauvegarde de dico.eol dans la variante
                fic_zephir = os.path.abspath(config.PATH_MODULES)+os.sep+str(self.id_mod)+os.sep+'variantes'+os.sep+str(self.id_var)+os.sep+'dico.eol'
                key = 'dico_ok'
            elif mode in ['migration','modif_migration']:
                fic_zephir = os.path.join(self.confdir,'migration.eol')
                key = 'migration_ok'
            dico_zeph.save(fic_zephir,encode=encode,force=force)
            if key != '':
                self.maj_params({key:1})
            self.check_md5conf()
        except:
            traceback.print_exc()
            raise IOError("serveur %s : erreur de sauvegarde de %s" % (self.id_s, os.path.basename(fic_zephir)))
        return ""
            
    def get_status(self):
        """renvoie le dernier état enregistré du serveur
        """
        try:
            assert self.status in range (5)
        except:
            return -1
        else:
            return self.status
    
    def set_status(self, status):
        """Met à jour l'état du serveur
        """
        self.status = int(status)
        cu = cx_pool.create()
        try:
            cu.execute("update serveurs set etat=%s where id=%s" % (self.status, self.id_s))
            cx_pool.commit(cu)
        except:
            traceback.print_exc()
            cx_pool.rollback(cu)
        return self.status

    def get_params(self, params=None):
        """renvoie le champ paramètres du serveur
        """
        try:
            if params is None:
                cu = cx_pool.create()
                try:
                    cu.execute("select params from serveurs where id=%s" % str(self.id_s))
                    data = cu.fetchone()
                    cx_pool.close(cu)
                except:
                    traceback.print_exc()
                    cx_pool.close(cu)
                params = {}
                try:
                    params = eval(data[0])
                    assert type(params) == dict and params.has_key('migration_ok')
                except:
                    # met à jour le champ param si il n'est pas encore renseigné
                    params = self.update_params(params)
                    # sauvegarde du champs params
                    self.save_params(params)
            else:
                if type(params) == str:
                    params = eval(params)

            # nombre de commandes et transferts uucp en attente
            id_uucp = self.rne+'-'+str(self.id_s)
            params['uucp_transfert']=0
            params['uucp_cmd']=0
            if os.path.isdir('/var/spool/uucp/%s/D.' % id_uucp):
                params['uucp_transfert']=len(os.listdir('/var/spool/uucp/%s/D.' % id_uucp))
            if os.path.isdir('/var/spool/uucp/%s/D.X' % id_uucp):
                params['uucp_cmd']=len(os.listdir('/var/spool/uucp/%s/D.X' % id_uucp))
            # si la configuration n'était pas présénte, on vérifie si c'est toujours le cas
            if params['config_ok'] == 0:
                if os.path.isfile(self.confdir+os.sep+'zephir.eol'):
                    params['config_ok']=1
            if params['dico_ok'] == 0:
                if os.path.isfile(self.confdir+os.sep+'dico.eol'):
                    params['dico_ok']=1
            if os.path.isfile(self.confdir+os.sep+'new_key.pub'):
                params['new_key']=1
            else:
                params['new_key']=0
            if self.version != 'creole1':
                params['migration_ok'] = 1
            elif os.path.isfile(self.confdir+os.sep+'migration.eol'):
                params['migration_ok'] = 0
            else:
                params['migration_ok'] = -1
            return params
        except:
            traceback.print_exc()
            return {}

    def edit_serveur(self, dico_modifs):
        # on vérifie que l'identifiant n'est pas modifié
        if 'id' in dico_modifs.keys():
            return 0, config.u("""l'identifiant ne peut pas être modifié""")
        # idem pour le rne et le module
        if 'rne' in dico_modifs.keys():
            return 0, config.u("""le rne ne peut pas être modifié""")
        # idem pour le rne et le module
        if ('module_actuel' in dico_modifs.keys()) or ('module_initial' in dico_modifs.keys()):
            return 0, config.u("""le module ne peut pas être modifié""")
        # construction de la requête SQL de modification
        if dico_modifs == {}:
            return 1, config.u("""aucune modification demandée""")
        else:
            # cas spécial : si on modifie le module et pas la variante, mettre variante 0 par défaut
            if ('module_actuel' in dico_modifs.keys()) and not ('variante' in dico_modifs.keys()):
                dico_modifs['variante']=0
            requete=["update serveurs set "]
            for cle in dico_modifs.keys():
                requete.append(str(cle))
                requete.append("=E'")
                requete.append(adbapi.safe(str(dico_modifs[cle])))
                requete.append("', ")
            query="".join(requete)[:-2]
            query += """ where id=%s""" % self.id_s
            # modification dans la base et appel à la fonction de mise à jour dans l'arborescence

        try:
            cu = cx_pool.create()
            cu.execute(query)
            cx_pool.commit(cu)
        except:
            return 0, config.u("""Erreur lors de la modification dans la base""")

        serveur_dir_ori = self.confdir
        if 'variante' in dico_modifs.keys():
            if int(dico_modifs['variante']) != int(self.id_var):
                # on loggue le changement de variante
                try:
                    cu = cx_pool.create()
                    cu.execute("""insert into log_serveur (id_serveur,date,type,message,etat) values (%s,'%s','ZEPHIR','Modification de la variante (%s -> %s)',0)""" % (int(self.id_s),str(time.ctime()),str(self.id_var),str(dico_modifs['variante'])))
                    cx_pool.commit(cu)
                except:
                    # on ne bloque pas si le log échoue
                    pass

            # mise à jour des liens vers la variante
            var_dir = os.path.abspath(config.PATH_MODULES)+os.sep+str(self.id_mod)+os.sep+'variantes'+os.sep+str(dico_modifs['variante'])
            try:
                os.unlink(serveur_dir_ori+os.sep+'dico.eol')
                os.unlink(serveur_dir_ori+os.sep+'droits_variante')
            except:
                pass

            retour = os.system('ln -s %s/dico.eol %s/dico.eol' % (var_dir, serveur_dir_ori))
            retour = os.system('ln -s %s/droits_zephir %s/droits_variante' % (var_dir, serveur_dir_ori))
            for rep in ['patchs','dicos','fichiers_perso','fichiers_zephir']:
                if retour != 0:
                    break
                if os.path.exists(serveur_dir_ori+'/%s/variante' % rep):
                    os.unlink(serveur_dir_ori+'/%s/variante' % rep)
                retour = os.system('ln -s %s/%s %s/%s/variante' % (var_dir, rep, serveur_dir_ori, rep))

            if retour != 0:
                # erreur lors de la création de l'arborescence
                return 0, config.u("""erreur de création des liens de variante""")

        self.update_data()
        # tout s'est bien passé (on peut toujours rêver)
        return 1, config.u('ok')

    def update_params(self,params_ori):
        # le champ params n'est pas initialisé
        if self.version == 'creole1':
            migration_ok = -1
        else:
            # pour les serveurs NG, on considère que la migration est déjà faite
            migration_ok = 1
        params = {'agents':1
                  ,'dico_ok':0
                  ,'config_ok':0
                  ,'migration_ok':migration_ok
                  ,'cle_ok':0
                  ,'new_key':0
                  #,'uucp_transfert':0
                  #,'uucp_cmd':0
                  ,'maj_ok':[-2,'']
                  ,'reboot_ok':[-2,'']
                  ,'service_restart_ok':[-2,'']
                  ,'query_maj':[-2,'']
                  ,'reconfigure_ok':[-2,'']
                  ,'configure_ok':[-2,'']
                  ,'sauvegarde_ok':[-2,'']
                  ,'lock_ok':[1,'']
                  ,'timeout':[1,'']
                  ,'md5s':[-1,'']
                  ,'last_log':'journal vide'
                }
        try:
            params['agents'] = params_ori['agents']
        except:
            # le serveur n'a pas encore d'info sur les agents
            pass
        # présence de zephir.eol et clé d'enregistrement
        if os.path.isfile(self.confdir+os.sep+'zephir.eol'):
            params['config_ok']=1
        if os.path.isfile(self.confdir+os.sep+'cle_publique'):
            params['cle_ok']=1
        if os.path.isfile(self.confdir+os.sep+'new_key.pub'):
            params['new_key']=1
        # vérification des logs pour maj et reconfigure
        query = """select type,etat,date,message from last_log_serveur where id_serveur = %s order by date desc,id desc limit 1""" % self.id_s
        cu = cx_pool.create()
        try:
            cu.execute(query)
            data = cu.fetchone()
            # on stocke la date du dernier log remonté
            if data is not None:
                params['last_log']=str(data[2])
            query = """select type,etat,date,message from last_log_serveur where id_serveur = %s and type != 'SURVEILLANCE' order by date desc,id desc""" % self.id_s
            cu.execute(query)
            data = cu.fetchall()
            cx_pool.close(cu)
            # on parse les logs
            not_found = ['maj','reconfigure','configure','lock','sauvegarde','reboot','service_restart','query_maj']
            for log in data:
                # on regarde si le serveur n'a pas remonté un bloquage des tâches zephir
                # (pb : procédure pour remonter la résolution du problème)
                if log[0] == 'LOCK':
                    if 'lock' in not_found:
                        not_found.remove('lock')
                        if log[1] == 1:
                            params['lock_ok'] = [2,str(log[2]),log[3]]
                # QUERY-MAJ : nombre de paquets à mettre à jour
                elif log[0] == 'QUERY-MAJ':
                    if self.version == 'creole1':
                        params['query_maj'] = [-2,""]
                    else:
                        if 'query_maj' in not_found:
                            not_found.remove('query_maj')
                            if log[1] == 0:
                                params['query_maj'] = [0,str(log[2])]
                                if 'paquets' in log[3]:
                                    try:
                                        nb_paquets = int(log[3].split()[0])
                                        params['query_maj'] = [nb_paquets,str(log[2])]
                                    except:
                                        params['query_maj'] = [-2,'erreur de lecture du résultat']
                            elif log[1] > 0:
                                # Erreur de Query-Auto
                                params['query_maj'] = [-1, log[3]]
                else:
                    for type_log in ['maj','configure','reconfigure','sauvegarde','reboot','service_restart']:
                        # état des actions
                        if log[0] == type_log.upper() and int(log[1]) != -2:
                            # on ne stocke que le dernier etat de chaque action
                            if type_log in not_found:
                                not_found.remove(type_log)
                                if log[1] == -1:
                                    params['%s_ok' % type_log] = [2,str(log[2]),log[3]]
                                elif log[1] > 0:
                                    params['%s_ok' % type_log] = [0,str(log[2]),log[3]]
                                elif log[1] == 0:
                                    params['%s_ok' % type_log] = [1,str(log[2]),log[3]]
                        break
                if not_found == []:
                    break
        except:
            traceback.print_exc()
            cx_pool.close(cu)
        return params

    def maj_params(self, param_updates):
        """modifie le champ param d'un serveur
        @param_updates : dictionnaire contenant les modifications
        """
        params_ori = self.get_params()
        params_ori.update(param_updates)
        self.save_params(params_ori)

    def save_params(self, params):
        params = adbapi.safe(str(params))
        query = """update serveurs set params=E'%s' where id=%s""" % (params, self.id_s)
        cu = cx_pool.create()
        try:
            cu.execute(query)
            cx_pool.commit(cu)
        except:
            traceback.print_exc()
            cx_pool.rollback(cu)

    def check_md5conf(self):
        # création de la liste des fichiers présents sur zephir
        modifs = []
        fics=[]
        files=md5files[self.module_version]
        # cas particulier : zephir.eol
        # creole 1 : suppression des lignes sans variables
        # creole 2 : valeurs précédentes ignorées
        fic_ori = os.path.join(self.confdir,'zephir.eol')
        fic_var = os.path.join(self.confdir,'variables.eol')
        if os.path.isfile(fic_ori):
            f_var = file(fic_var,'w')
            for line in file(fic_ori).readlines():
                if self.module_version == 1:
                    if "@@" in line:
                        f_var.write(line)
                else:
                    if not "valprec =" in line:
                        f_var.write(line)
            f_var.close()
        for src, dst, pattern in files:
            if os.path.isdir(os.path.join(self.confdir,dst)):
                for fic in os.listdir(os.path.join(self.confdir,dst)):
                    if os.path.isfile(os.path.join(self.confdir,dst,fic)):
                        if pattern == None or fic.endswith(pattern):
                            fics.append(os.path.join(dst,fic))
            else:
                fics.append(dst)
        md5s = []
        md5file = os.path.join(os.path.abspath(config.PATH_ZEPHIR),'data','config%s.md5' % self.id_s)
        if os.path.isfile(md5file):
            # liste des fichiers remontés par le serveur
            data = file(md5file).read().strip().split('\n')
            # comparaison
            for line in data:
                fic = line.split()[1]
                if fic not in fics:
                    modifs.append("absent sur zephir : %s" % fic)
                else:
                    md5s.append(fic)
            for fic in fics:
                if fic not in md5s:
                    # erreur : fichier non présent sur la machine distante
                    modifs.append("non envoyé : %s" % fic)
            # test du contenu des fichiers
            cmd_md5 = getProcessOutputAndValue("/usr/bin/md5sum",
                                        args = ["-c",md5file],
                                        path = self.confdir,
                                        env = {'LC_ALL': 'C'})
            cmd_md5.addCallback(self._check_md5_res, md5s, modifs)
            cmd_md5.addErrback(self._check_md5_res, md5s, modifs)
        else:
            # pas d'infos md5 pour ce serveur
            self.maj_params({'md5s':[-1,""]})
            self.edit_serveur({'md5s':-1})

    def _check_md5_res(self, result, md5s, modifs):
        # on effectue une vérification md5 des fichiers de configuration si disponible
        out, err, code = result
        self.edit_serveur({'md5s':1})
        self.maj_params({'md5s':[1,""]})
        for fic in md5s:
            if not "%s: OK" % fic in out.split('\n'):
                if fic == 'variables.eol':
                    # cas particulier des variables creole 1, on indique une modification sur zephir.eol
                    modifs.append("contenu modifié : zephir.eol")
                else:
                    modifs.append("contenu modifié : %s" % fic)
                md5_ok = False
        if modifs != []:
            self.edit_serveur({'md5s':0})
            self.maj_params({'md5s':[0,";".join(modifs).strip()]})
   
    def check_maj_status(self, show_installed = False):
        """vérifie l'état de mise à jour d'un serveur
        """
        # liste des paquets remontés par le serveur
        pkgs = []
        liste_pkgs = []
        if self.version == 'creole1':
            return liste_pkgs
        pkg_file = os.path.join(os.path.abspath(config.PATH_ZEPHIR),'data','packages%s.list' % self.id_s)
        if os.path.isfile(pkg_file):
            for line in file(pkg_file):
                pkgs.append(line.split())
        # on regarde le type de mise à jour configurée (minimale/complète)
        dico = self.get_config()
        type_maj = dico.get_value('type_maj')[0]
        # on compare les paquets installés avec ceux disponibles sur le serveur de maj
        version = self.module_version
        liste_pkgs = maj_checker.check_packages(pkgs,version,type_maj,show_installed)
        if show_installed == False:
            if len(liste_pkgs) > 0:
                self.edit_serveur({'maj':len(liste_pkgs)})
                self.maj_params({'query_maj':[len(liste_pkgs),time.ctime()]})
            else:
                self.edit_serveur({'maj':0})
                self.maj_params({'query_maj':[0,time.ctime()]})
        return liste_pkgs

    def regen_key(self):
        """prépare une nouvelle clé ssh pour la connexion uucp à zephir
        """
        new_key = os.path.join(self.confdir, 'new_key')
        if os.path.exists(new_key):
            os.unlink(new_key)
        if os.path.exists("%s.pub" % new_key):
            os.unlink("%s.pub" % new_key)
        res = os.system("""/usr/bin/ssh-keygen -N "" -b 1024 -t rsa -f %s -C uucp@%s""" % (new_key, config.ADRESSE_ZEPHIR))
        return res

    def get_key(self, old_key, new_key):
        """envoie une nouvelle clé à un serveur.
        les 2 clés publiques (ancienne et nouvelles) doivent être passées en paramètre.
        """
        try:
            old_key_path = os.path.join(self.confdir, 'cle_publique')
            new_key_path = os.path.join(self.confdir, 'new_key.pub')
            new_key_priv_path = os.path.join(self.confdir, 'new_key')
            old_data = file(old_key_path).read().strip()
            old_data = base64.encodestring(old_data[old_data.index('ssh-rsa'):])
            if old_data == old_key:
                # la clé correspond, on renvoie les nouvelles clés
                data_keys = []
                new_data = file(new_key_path).read().strip()
                new_data = base64.encodestring(new_data[new_data.index('ssh-rsa'):])
                if new_data != new_key:
                    return 0, "nouvelle clé publique non reconnue"
                new_data_priv = base64.encodestring(file(new_key_priv_path).read().strip())
                os.unlink(new_key_priv_path)
                return 1, new_data_priv
            else:
                return 0, "ancienne clé publique non reconnue"
        except:
            return 0, "erreur de récupération des données"

    def get_last_contact(self):
        """renvoie la date de dernier contact avec le serveur
        """
        cu = cx_pool.create()
        try:
            cu.execute("select last_contact from serveurs where id=%s" % str(self.id_s))
            data = cu.fetchone()
            cx_pool.close(cu)
        except:
            traceback.print_exc()
            cx_pool.close(cu)
            return 0.0
        return float(data[0])

    def migrate_config(self,module_dest,variante_dest,mode="migration"):
        """crée un dictionnaire creole2 et importe les valeurs du serveur"""
        assert self.version == 'creole1'
        assert int(module_dest) >= 1
        assert int(variante_dest) >= 1
        # chemin vers les dictionnaires du module
        creoledirs=[os.path.join(os.path.abspath(config.PATH_MODULES),str(module_dest),'variantes',str(variante_dest),'dicos')]
        creoledirs.append(os.path.join(os.path.abspath(config.PATH_MODULES),str(module_dest),'dicos'))
        # création du dictionnaire 
        dico = ZephirDict(creoledirs, self.confdir, mode)
        # on stocke la variable choisie dans le répertoire de conf
        f_var = open(os.path.join(self.confdir, 'variante_migration'), 'w')
        f_var.write(str(variante_dest))
        f_var.close()
        return dico

    def update_module(self, data, id_new_mod, id_variante, hw_infos, cred_user):
        """fonction de mise à jour du module actuel (upgrade entre deux distribution)
        """


    def migrate_data(self, check=False):
        """fonction de récupération des données d'un serveur migré
        check : vérifie seulement si des données sont migrables sur ce serveur
        """
        # fichiers à migrer : fichiers par défaut
        migration_files = {}
        module = self.module[:self.module.rindex('-')]
        if module in config.migration_files:
            migration_files.update(config.migration_files[module])
        # définitions supplémentaires
        if module in migration_perso:
            for section in migration_perso[module].keys():
                migration_files[section].extend(migration_perso[module][section])
        # mode check : indique si des fichiers sont gérés pour ce serveur
        if check == True:
            if len(migration_files.keys()) > 0:
                return 1, True
            else:
                return 1, False
        # liste des fichiers à copier
        if not len(migration_files.keys()) > 0:
            return 1, config.u("""pas de fichiers à migrer""")
        dir_serv = os.path.join(self.confdir,'fichiers_zephir')
        dir_bak = os.path.join(self.confdir+'-backup','fichiers_zephir')
        if not os.path.isdir(dir_bak):
            return 0, config.u("répertoire backup de migration non trouvé")
        not_found = []
        errors = []
        # copie des fichiers avec conversion de l'encoding si besoin
        for src, dst, convert in migration_files['files']:
            fic_src = os.path.join(dir_bak,src)
            fic_dst = os.path.join(dir_serv,dst)
            if os.path.isdir(fic_src):
                # répertoire
                os.path.walk(fic_src, self.tree_copy, (fic_src, fic_dst, convert, errors, migration_files))
            elif os.path.isfile(fic_src):
                self.copy_fic(fic_src, fic_dst, convert, errors)
            else:
                not_found.append(src)
        # application des droits
        droits_zephir = []
        dests=[]
        for dest, options, user, group, mode in migration_files['rights']:
            if dest.endswith("*"):
                # cas "repertoire/*" : on recherche tous les fichiers
                dests = glob(os.path.join(dir_serv,dest))
            else:
                if os.path.exists(os.path.join(dir_serv,dest)):
                    dests = [os.path.join(dir_serv,dest)]
            for fic_dest in dests:
                if os.path.basename(fic_dest) not in migration_files['exclude']:
                    cmd_rights = 'chmod %s %s %s;chown %s %s.%s %s' % (options, mode, fic_dest, options, user, group, fic_dest)
                    res = os.system(cmd_rights)
                    if res != 0:
                        errors.append('application des droits impossible : %s' % fic_dest)
                    else:
                        # ajout de la ligne pour droits_zephir
                        droits_zephir.append("%s#%s#%s#%s#%s" % (fic_dest[fic_dest.index('fichiers_zephir'):],mode,user,group,'-R' in options))

        # création du fichier droits_zephir
        fic_droits = open(os.path.join(self.confdir, 'droits_zephir'),'w')
        fic_droits.write('\n'.join(droits_zephir))
        fic_droits.close()
        # mise en place du fichier fichiers_zephir (copie de fichiers_module + destinations personnalisées)
        module_file = '/usr/share/zephir/dictionnaires/fichiers_%s' % self.module
        serveur_file = os.path.join(self.confdir,'fichiers_zephir','fichiers_zephir')
        if os.path.isfile(module_file):
            # lecture du fichier du module
            data = file(module_file).read().strip()
        if module in destinations_perso:
            data += "\n# fichiers ajoutés lors de la migration Eole 1 vers Eole NG"
            for dest in destinations_perso[module]:
                data += "\n%s" % dest

        # écriture en utilisant les templates définis dans config.py
        final_data = "%s\n%s\n%%%%\n%s" % (config.FILE_SECTION,data,config.RPM_SECTION)

        # écriture du fichier final
        dests = file(serveur_file,'w')
        dests.write(final_data)
        dests.close()

        return 1, config.u((not_found,errors))

    def variante_migration(self):
        path_var = os.path.join(self.confdir, 'variante_migration')
        if os.path.isfile(path_var):
            return open(path_var).read().strip()
        else:
            return ''

    def tree_copy(self, args, dirname, fnames):
        fic_src, fic_dst, convert, errors, migration_files = args
        if not os.path.isdir(fic_dst):
            os.makedirs(fic_dst)
        for fic in fnames:
            if fic not in migration_files['exclude']:
                src = os.path.join(dirname, fic)
                dst = src.replace(fic_src, fic_dst)
                if os.path.exists(src):
                    self.copy_fic(src, dst, convert, errors)

    def copy_fic(self, src, dst, convert, errors):
        """copie du fichier src sur dst
        convert: si True, convertit le fichier d'ISO-8859-1 vers UTF-8
        """
        if convert == True:
            cmd = '/usr/bin/iconv -f ISO-8859-1 -t UTF-8 -o "%s" "%s"' % (dst, src)
        else:
            cmd = '/bin/cp -f "%s" "%s"' % (src, dst)
        res = os.system(cmd)
        if res != 0:
            print "erreur lors de la migration des données, la commande suivante a échoué :"
            print cmd
            errors.append(src)


class ServeurPool(dict):
    """dictionnaire des serveurs existants gérant les autorisations d'accès
    """

    def __init__(self):

        self._groupes_serveurs = {}
        self._restrictions = {}
        self.stats = {}
        self.update_groupes()
        # dictionnaire des variantes et leur module
        self._mod_var = self.get_mod_var()
        try:
            self.update_auths()
        except:
            # pas encore de table de restriction sur cette version
            pass

    def update_groupes(self):
        cu = cx_pool.create()
        # récupération des groupes de serveurs
        try:
            cu.execute("select id, libelle, serveurs from groupes_serveurs")
            data = cu.fetchall()
            for groupe in data:
                id = int(groupe[0])
                libelle = str(groupe[1])
                serveurs = eval(groupe[2])
                timestamp = time.time()
                date_creat = timestamp
                if self._groupes_serveurs.has_key(id):
                    date_creat = self._groupes_serveurs[id][2][0]
                self._groupes_serveurs[id] = [libelle, serveurs, [date_creat, timestamp]]
            cx_pool.close(cu)
        except:
            traceback.print_exc()
            cx_pool.close(cu)
       
    def get_mod_var(self):
        """récupére les variantes et leur module"""
        cu = cx_pool.create()
        try:
            cu.execute("select id,module from variantes")
            data = cu.fetchall()
            cx_pool.close(cu)
        except:
            data = []
            traceback.print_exc()
            cx_pool.close(cu)
        return dict(data)
       
    def update_auths(self, user=None):
        if user != None:
            user_clause = " where login = '%s'" % user
        else:
            user_clause = ""

        cu = cx_pool.create()
        try:
            # récupération des types de contraintes spécifiées
            cu.execute("select login, id_res, type_res from restrictions %s" % user_clause)
            data = cu.fetchall()
            for restriction in data:
                login, id_res, type_res = restriction
                if not self._restrictions.has_key(login):
                    self._restrictions[login] = {}
                if self._restrictions[login].has_key(type_res):
                    self._restrictions[login][type_res].append(id_res)
                else:
                    self._restrictions[login][type_res] = [id_res]
            cx_pool.close(cu)
        except:
            traceback.print_exc()
            cx_pool.close(cu)

    def add_restriction(self, credential, type_res, id_res):
        """ajoute une restriction pour un utilisateur et une ressource donnés
        type_res : nature des objets à restreindre
        id_res ; identifiant de l'objet autorisé
        les différents types reconnus sont : 'rne, group, id_mod, id_var' et les attributs
        de la classe serveur (id_s, module, version, ...)
        """
        # ajout dans la base
        try:
            assert str(id_res) in self._restrictions[credential][type_res]
        except: 
            cu = cx_pool.create()
            # validation de l'existence
            libelle, table, field = config.type_res_label[type_res]
            if field in ('rne','libelle'):
                # données de type str
                valid_query = "select * from %s where %s ilike '%s'" % (table, field, str(id_res))
            else:
                # type entier
                valid_query = "select * from %s where %s = %s" % (table, field, str(id_res))
            try:
                cu.execute(valid_query)
                # pas de données correspondante trouvée
                if cu.fetchone() == None:
                    raise ValueError('identifiant de ressource inconnu')
            except Exception, e:
                cx_pool.close(cu)
                return False
            try:
                cu.execute("""insert into restrictions (login, type_res, id_res) values ('%s','%s','%s')""" % (credential, type_res, str(id_res)))
            except:
                # erreur d'insertion dans la base
                cx_pool.rollback(cu)
                return False
            cx_pool.commit(cu)
            if not self._restrictions.has_key(credential):
                self._restrictions[credential] = {}
            if not self._restrictions[credential].has_key(type_res):
                self._restrictions[credential][type_res] = []
            self._restrictions[credential][type_res].append(str(id_res))
        else:
            # restriction déjà existante
            pass
        return True

    def get_restrictions(self, credential, type_res=None):
        """renvoie la liste des restrictions d'un utilisateur
        """
        if not self._restrictions.has_key(credential):
            return []
        if type_res == None:
            # toutes les restrictions : dictionnaire
            return self._restrictions[credential]
        else:
            # si restrictions d'un type précis, on renvoie une liste de ressources
            if self._restrictions[credential].has_key(type_res):
                return self._restrictions[credential][type_res]
            else:
                return []

    def del_restriction(self, credential, type_res, id_res):
        """enleve une restriction d'un utilisateur
        """
        try:
            liste_res = self._restrictions[credential][type_res]
            assert str(id_res) in liste_res
        except:
            # restriction non existante
            return False
        # suppression dans la base
        cu = cx_pool.create()
        try:
            cu.execute("""delete from restrictions where login='%s' and type_res='%s' and id_res='%s'""" \
                    % (credential, type_res, str(id_res)))
            cx_pool.commit(cu)
        except:
            traceback.print_exc()
            cx_pool.rollback(cu)
            return False
        # supression en mémoire
        self._restrictions[credential][type_res].remove(str(id_res))
        if self._restrictions[credential][type_res] == []:
            del(self._restrictions[credential][type_res])
        return True

    def get_file_perms(self, data_dir, filepath=""):
        """renvoie les informations de permissions associées à un fichier
        """
        # lecture des informations sur le fichier si disponibles
        permsfile = data_dir+os.sep+'droits_zephir'
        if os.path.isfile(permsfile):
            f_rights = file(permsfile)
            data = f_rights.read().strip().split('\n')
            f_rights.close()
        else:
            data = []
        mode = user = group = ""
        recursive = False
        if filepath != "":
            result = {filepath:[mode, user, group, recursive]}
        else:
            result = {}
        for line in data:
            if line.startswith(filepath+'#') or filepath == "":
                try:
                    filename, mode, user, group, recursive = line.split('#')
                    if recursive != '':
                        recursive = eval(recursive)
                    result[filename] = [mode, user, group, recursive]
                    if filepath != "":
                        break
                except:
                    # fichier vide ?
                    pass
        return result

    def set_file_perms(self, rights, data_dir):
        """enregistre les informations de permissions associées à un(des) fichier(s)
        @param data_dir: chemin ou trouver le fichier droits_zephir
        """
        # lecture des données existantes
        permsfile = data_dir+os.sep+'droits_zephir'
        data = self.get_file_perms(data_dir)
        data.update(rights)
        # stockage des informations
        lines = []
        for filepath in data.keys():
            # vérification des données
            lines.append("%s#%s#%s#%s#%s" % (filepath, data[filepath][0], data[filepath][1], data[filepath][2],data[filepath][3]))
        # écriture des informations
        f_rights = file(permsfile,'w')
        f_rights.write('\n'.join(lines))
        f_rights.close()
        return True

    def del_file_perms(self, data_dir, filepath="", recurse=False):
        """supprime les informations de permissions associées à un fichier (ou tous)
        """
        # lecture des informations sur le fichier si disponibles
        permsfile = data_dir+os.sep+'droits_zephir'
        if not os.path.exists(permsfile):
            # pas de permissions exitantes ou mauvais chemin
            return False
        if filepath != "":
            f_rights = file(permsfile)
            data = f_rights.read().strip().split('\n')
            f_rights.close()
            if recurse == True:
                search_pattern = filepath
            else:
                search_pattern = filepath + '#'
            for line in data:
                if line.startswith(search_pattern):
                    data.remove(line)
        else:
            data = []
        # écriture des informations
        try:
            f_rights = file(permsfile,'w')
            f_rights.write('\n'.join(data))
            f_rights.close()
        except:
            return False
        return True

    def check_serv_credential(self, credential, key):
        # vérification des restrictions sur les attributs du serveur
        # (id_s, rne, id_mod, variante)
        serv = dict.get(self,key)
        if self._restrictions.has_key(credential):
            for type_res, res in self._restrictions[credential].items():
                if res == []:
                    # ce cas ne devrait pas arriver car si on passe toujours par del_restriction
                    continue
                res_ok = True
                # si la ressource n'est pas autorisée
                # cas particuliers : rne et groupes
                if type_res == 'rne':
                    rne_ok = False
                    for rne in res:
                        # cas des rne avec wildcard (%)
                        if rne.endswith('%'):
                            if serv.rne.startswith(rne.split('%')[0]):
                                # ce rne correspond à l'expression
                                rne_ok = True
                                break
                    if not rne_ok:
                        if not serv.rne in res:
                            res_ok = False
                elif type_res == 'groupe':
                    # test d'appartenance à un groupe autorisé
                    serv_in_groupes = False
                    for groupe in res:
                        if serv.id_s in self._groupes_serveurs[int(groupe)][1]:
                            serv_in_groupes = True
                            break
                    if serv_in_groupes == False:
                        res_ok = False
                else:
                    # test attribut correspondant du serveur dans les valeurs autorisées
                    if not str(getattr(serv,type_res)) in [str(i) for i in res]:
                        res_ok = False
                if not res_ok:
                    # on interdit l'accès
                    raise ResourceAuthError("unauthorized resource  : %s" % key)

    def check_gr_credential(self, credential, groupe):
        # validation des droits d'accès à un groupe
        if self._restrictions.has_key(credential):
            if self._restrictions[credential].has_key('groupe'):
                if self._restrictions[credential]['groupe'] != []:
                    if str(groupe) not in self._restrictions[credential]['groupe']:
                        # accès refusé
                        # FIXME vérification suffisante ou regarder les serveurs ou rne du groupe ?
                        raise ResourceAuthError("unauthorized resource  : %s" % groupe)

    def check_mod_credential(self, credential, id_mod):
        # validation des droits d'accès à un module
        if self._restrictions.has_key(credential):
            if self._restrictions[credential].has_key('id_mod'):
                if self._restrictions[credential]['id_mod'] != []:
                    if str(id_mod) not in self._restrictions[credential]['id_mod']:
                        # accès refusé
                        raise ResourceAuthError("unauthorized module  : %s" % id_mod)

    def check_var_credential(self, credential, id_var):
        # validation des droits d'accès à une variante
        if self._restrictions.has_key(credential):
            try:
                var_mod = str(self._mod_var[int(id_var)])
            except:
                # on met à jour si la variante n'est pas trouvée
                self._mod_var = self.get_mod_var()
                var_mod = str(self._mod_var[int(id_var)])
            # on commence par vérifier le module auquel la variante appartient
            if self._restrictions[credential].has_key('id_mod'):
                if self._restrictions[credential]['id_mod'] != []:
                    if var_mod not in self._restrictions[credential]['id_mod']:
                        raise ResourceAuthError("accès au module %s interdit" % var_mod)
            # vérification de la variante elle-même
            if self._restrictions[credential].has_key('id_var'):
                if self._restrictions[credential]['id_var'] != []:
                    if str(id_var) not in self._restrictions[credential]['id_var']:
                        # accès refusé
                        raise ResourceAuthError("accès à la variante %s interdit" % id_var)

    def check_etab_credential(self, credential, rne):
        # validation des droits d'accès à un établissement
        rne_ok = True
        if self._restrictions.has_key(credential):
            if self._restrictions[credential].has_key('rne'):
                if self._restrictions[credential]['rne'] != []:
                    if str(rne) not in self._restrictions[credential]['rne']:
                        rne_ok = False
                        for etab in self._restrictions[credential]['rne']:
                            if etab.endswith('%'):
                                if str(rne).startswith(etab.split('%')[0]):
                                    rne_ok = True
        if not rne_ok:
            # accès refusé
            raise ResourceAuthError("unauthorized module  : %s" % rne)

    def get(self, credential, key):
        if key in self.keys():
            self.check_serv_credential(credential, key)
        else:
            raise KeyError, "serveur inexistant"
        return dict.get(self,key)

    def get_alertes(self, credential):
        """renvoie la liste des serveurs en alerte
        """
        servs=[]
        for id_serv in self.keys():
            try:
                serv = self.get(credential, int(id_serv))
            except ResourceAuthError:
                # accès à ce serveur non autorisé
                continue
            if serv.get_status() not in [1, -1]:
                servs.append(serv)
        return servs

    def get_migration_status(self, credential):
        """renvoie la liste des serveurs en cours de migration/non migrés
        """
        servs=[[],[]]
        for id_serv in self.keys():
            try:
                serv = self.get(credential, int(id_serv))
            except ResourceAuthError:
                # accès à ce serveur non autorisé
                continue
            params = serv.get_params()
            if params['migration_ok'] == 0:
                servs[0].append(serv)
            elif params['migration_ok'] == -1:
                servs[1].append(serv)
        return servs

    def check_serveurs(self, credential, serveurs, last_check=None):
        """vérifie si des serveurs ont été modifiés
        @param serveurs : dictionnaires {idserveur:timestamp}
        @param last_check : Si != None, on renvoie les serveurs créés après cette date (timestamp)
        @return : liste d'id de serveurs
        """
        modifs = {}
        serv_pool = []
        for id_serv in [str(cle) for cle in self.keys()]:
            try:
                serv = self.get(credential, int(id_serv))
            except ResourceAuthError:
                # serveur non existant ou non accessible
                print "server %s not authorized" % id_serv
                if serveurs.has_key(id_serv):
                    modifs[id_serv] = "deleted"
            else:
                serv_pool.append(str(serv.id_s))
                if serveurs.has_key(id_serv):
                    if serv.modified > serveurs[id_serv]:
                        # serveur modifié
                        modifs[id_serv] = serv.modified
                elif last_check != None:
                    if serv.created > last_check:
                        # si le serveur a été créé après la dernière mise à jour du client
                        # on l'ajoute comme nouveau serveur
                        modifs[id_serv] = serv.modified

        for id_serv in serveurs.keys():
            if id_serv not in serv_pool:
                modifs[id_serv] = "deleted"
            
        return modifs

    def check_groupes(self, credential, groupes, last_check=None):
        """vérifie si des groupes ont été modifiés
        @param groupes : dictionnaires {idgroupe:timestamp}
        @param last_check : Si != None, on renvoie les serveurs créés après cette date (timestamp)
        @return : dictionnaire  {id_gr:[libelle, serveurs, [date_creat, date_modif]]}
        """
        modifs = {}

        for id_gr, data in self._groupes_serveurs.items():
            try:
                group = self.get_groupes(credential,id_gr)
            except KeyError, ResourceAuthError:
                # groupe non existant ou non accessible
                print "group %s no longer exists or not authorized" % id_gr
                modifs[str(id_gr)] = "deleted"
            else:
                # la clé des dictionnaires doit être une chaine pour xmlrpc
                id_gr = str(id_gr)
                if groupes.has_key(id_gr):
                    if data[2][1] > groupes[id_gr]:
                        # groupe modifié
                        modifs[id_gr] = data
                elif last_check != None:
                    if data[2][0] > last_check:
                        # nouveau groupe
                        modifs[id_gr] = data

        return modifs

    def add_groupe(self, credential, libelle, serveurs):
        query = """insert into groupes_serveurs (libelle,serveurs) values ('%s', '%s')""" % (libelle,str(serveurs))
        cu = cx_pool.create()
        try:
            cu.execute(query)
        except:
            traceback.print_exc()
            cx_pool.rollback(cu)
            return False
        cx_pool.commit(cu)
        self.update_groupes()
        # ajout automatique du groupe pour l'utilisateur l'ayant créé si il a des restrictions de groupes
        if self._restrictions.has_key(credential):
            if self._restrictions[credential].has_key('groupe'):
                # on recherche l'id du groupe ajouté
                for id_gr in self._groupes_serveurs.keys():
                    if self._groupes_serveurs[id_gr][0] == libelle:
                        self.add_restriction(credential,'groupe',str(id_gr))
        return True

    def get_groupes(self, credential, id_groupe=None, alertes=False):
        if id_groupe != None:
            groupes = [int(id_groupe)]
        else:
            groupes = self._groupes_serveurs.keys()
        # vérification des restrictions sur les groupes
        res = []
        for groupe in groupes:
            try:
                self.check_gr_credential(credential, groupe)
            except:
                # groupe non autorisé, on passe au suivant
                continue
            libelle, serveurs, timestamps = self._groupes_serveurs[groupe]
            # sinon on ajoute le groupe à la liste
            res.append([int(groupe), libelle, serveurs])
        return res

    def edit_groupe(self, credential, id_groupe, libelle, serveurs):
        # vérification des droits sur le groupe
        id_groupe = int(id_groupe)
        self.check_gr_credential(credential, id_groupe)
        # mise à jour de la base
        cu = cx_pool.create()
        query = """update groupes_serveurs set libelle='%s', serveurs='%s' where id=%s""" % (libelle,str(serveurs),id_groupe)
        try:
            cu.execute(query)
        except:
            traceback.print_exc()
            cx_pool.rollback(cu)
            return False
        cx_pool.commit(cu)
        # mise à jour interne
        date_creat = self._groupes_serveurs[id_groupe][2][0]
        self._groupes_serveurs[id_groupe] = [libelle, serveurs, [date_creat,time.time()]]
        return True

    def del_groupe(self, credential, id_groupe):
        # vérification des droits sur le groupe
        id_groupe = int(id_groupe)
        self.check_gr_credential(credential, id_groupe)
        # on supprime le groupe des groupes surveillés par les utilisateurs
        d = {'id':id_groupe}
        # recherche des utilisateurs surveillant le groupe
        rech_groupe = """groupes like '[%(id)s]' or groupes like '[%%%(id)s,%%]' or groupes like '%%, %(id)s]'""" % d
        cursor = cx_pool.create()
        try:
            cursor.execute("""select login,groupes from users where %s""" % rech_groupe)
            data=cursor.fetchall()
            # pour chaque utilisateur
            for user in data:
                # modification de la liste des groupes
                groupes = eval(user[1])
                if id_groupe in groupes:
                    groupes.remove(id_groupe)
                    sql_update = """update users set groupes='%s' where login='%s'""" % (str(groupes),user[0])
                    cursor.execute(sql_update)
            # suppression du groupe dans la base
            cursor.execute("""delete from groupes_serveurs where id = %s""" % str(id_groupe))
            cx_pool.commit(cursor)
        except:
            traceback.print_exc()
            cx_pool.rollback(cursor)
            return False
        # suppression du groupe en mémoire
        del(self._groupes_serveurs[id_groupe])
        return True


    def extend_groupe(self, credential, id_groupe, serveurs):
        # vérification des droits sur le groupe
        id_groupe = int(id_groupe)
        self.check_gr_credential(credential, id_groupe)
        # extension des serveurs du groupe
        # dans le dictionnaire interne
        for serv in serveurs:
            if serv in self._groupes_serveurs[id_groupe][1]:
                serveurs.remove(serv)
        self._groupes_serveurs[id_groupe][1].extend(serveurs)
        self._groupes_serveurs[id_groupe][2] = [time.time(),time.time()]
        cu = cx_pool.create()
        # dans la base de données
        try:
            query="""update groupes_serveurs set serveurs='%s' where id=%s""" % (str(self._groupes_serveurs[id_groupe][1]),id_groupe)
            cu.execute(query)
            cx_pool.commit(cu)
        except:
            traceback.print_exc()
            cx_pool.rollback(cu)
        
    def add_serveur(self, credential, rne, libelle, materiel, processeur, disque_dur, date_install, installateur, tel, remarques, module_initial, module_actuel, variante, timestamp_serveur, timeout):
        # on vérifie si l'accès au rne , module et variante est permis
        if self._restrictions.has_key(credential):
            if self._restrictions[credential].has_key('rne'):
                self.check_etab_credential(credential, rne)
            if self._restrictions[credential].has_key('id_mod'):
                if module_actuel not in self._restrictions[credential]['id_mod']:
                    raise ResourceAuthError("unauthorized resource %s" % module_actuel)
            if self._restrictions[credential].has_key('variante'):
                cu = cx_pool.create()
                cu.execute('select libelle from variantes where id=%s' % variante)
                libel_var = cu.fetchone()[0]
                cx_pool.close(cu)
                # dans le cas d'une variante standard, on l'autorise toujours
                if libel_var != 'standard' and variante not in self._restrictions[credential]['variante']:
                    raise ResourceAuthError("unauthorized resource %s" % variante)
        # insertion dans la base de données
        cu = cx_pool.create()
        query = """insert into serveurs (rne,libelle,materiel,processeur,disque_dur,date_install,installateur,tel,remarques,module_initial,module_actuel,variante,timestamp,timeout) values ('%s',E'%s',E'%s',E'%s',E'%s',E'%s',E'%s','%s',E'%s',%s,%s,%s,'%s',%s)""" \
        % (rne,adbapi.safe(libelle),adbapi.safe(materiel),adbapi.safe(processeur),adbapi.safe(disque_dur),adbapi.safe(date_install),adbapi.safe(installateur),tel,adbapi.safe(remarques),module_initial,module_actuel,variante,timestamp_serveur,timeout)
        cu.execute(query)
        # récupération de l'identifiant attribué par la base
        query = """select id from serveurs where rne='%s' and libelle=E'%s' and date_install=E'%s' and installateur=E'%s' and remarques=E'%s' and module_actuel=%s and variante=%s and timestamp='%s'""" %  (rne, adbapi.safe(libelle), adbapi.safe(date_install), adbapi.safe(installateur), adbapi.safe(remarques), module_actuel, variante, timestamp_serveur)
        cu.execute(query)
        id_serveur = int(cu.fetchone()[0])
        self[id_serveur] = Serveur(id_serveur,cu)
        cx_pool.commit(cu)
        return id_serveur

    def del_serveur(self, credential, id_serveur):
        """Supression d'un serveur de la base
        """
        # vérification des restrictions
        self.check_serv_credential(credential, id_serveur)
        cu = cx_pool.create()
        # on supprime le serveur dans la base
        try:
            cu.execute("""delete from serveurs where id=%s""" % str(id_serveur))
            cx_pool.commit(cu)
            # supression interne
            del(self[id_serveur])
        except:
            traceback.print_exc()
            cx_pool.rollback(cu)

    def edit_serveur(self, id_serveur, dico_modifs):
        # modification d'un serveur dans la base
        serv = self[id_serveur]
        return serv.edit_serveur(dico_modifs)


    def update_contact(self, id_serveur):
        """mise à jour de la date de contact d'un serveur"""
        id_serveur = int(id_serveur)
        cu = cx_pool.create()
        try:
            # on met à jour le serveur dans la base
            query = """update serveurs set last_contact='%s' where id=%s""" % (str(time.time()),id_serveur)
            cu.execute(query)
            cx_pool.commit(cu)
            # mise à jour des statistiques
            if int(id_serveur) in self.stats['no_contact']:
                self.stats['no_contact'].remove(int(id_serveur))
        except:
            traceback.print_exc()
            cx_pool.rollback(cu)


def serveur_pool():
    """crée un pool de tous les serveurs existants
    """
    # création du pool de serveurs en mémoire
    pool=ServeurPool()
    cu = cx_pool.create()
    try:
        cu.execute("select id, rne, libelle, module_actuel, variante, timeout, etat, md5s, maj, last_contact from serveurs")
        data = cu.fetchall()
        nbserv = len(data)
        no_contact = []
        # Instanciation des serveurs
        for serv in data:
            # liste des serveurs n'ayant jamais contacté zephir
            if serv[9] == None:
                no_contact.append(int(serv[0]))
            # nombre total de serveurs
            pool[serv[0]] = Serveur(serv[0], cu, serv[1:2])
            pool[serv[0]].update_data(cu, serv[2:9])
        pool.stats['no_contact'] = no_contact
        pool.stats['nb_serv'] = len(data)
        cx_pool.close(cu)
    except:
        traceback.print_exc()
        cx_pool.close(cu)
    return pool

# utility functions

text_characters = "".join(map(chr, range(32, 255)) + list("\n\r\t\b"))
_null_trans = string.maketrans("", "")

def istextfile(filename, blocksize = 1024):
    return istext(open(filename).read(blocksize))

def istext(s):
    if "\0" in s:
        return 0
    
    if not s:  # Empty files are considered text
        return 1

    # Get the non-text characters (maps a character to itself then
    # use the 'remove' option to get rid of the text characters.)
    t = s.translate(_null_trans, text_characters)

    # If more than 10% non-text characters, then
    # this is considered a binary file
    if len(t)/len(s) > 0.10:
        return 0
    return 1

