# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# modules_rpc.py
#  
# fonctions xmlrpc pour la gestion des modules dans Zephir
#       
###########################################################################
"""module de gestion des modules Eole
"""
from twisted.python import log
from twisted.internet import reactor
from zephir.backend.db_utils import *
from zephir.backend import config
from zephir.backend.config import u
from zephir.backend.xmlrpceole import XMLRPCEole as XMLRPC
from zephir.backend.lib_backend import ResourceAuthError, istextfile, cx_pool
from zephir.utils.creolewrap import ZephirDict

# import relatifs aux tables
from twisted.enterprise import adbapi
import sys,os,shutil,md5,time,dico,base64,ConfigParser, traceback
from zephir.eolerpclib import xmlrpclib
from creole.cfgparser import EoleDict, eosfunc

class RPCModules(XMLRPC):
    """serveur XMLRPC zephir pour la gestion des modules Eole
    """

    def __init__(self,parent,bdd='zephir-parc'):
        self.dbpool = db_connect()
        self.dbpool.noisy = 0
        self.parent = parent
        # mise à jour des modules si besoin
        XMLRPC.__init__(self)

    def startup(self):
        self.xmlrpc_update_modules()

    def xmlrpc_update_modules(self,cred_user=''):
        """vérification de la liste des modules"""
        cursor = cx_pool.create()
        try:
            cursor.execute("""select libelle from modules""")
            modules_actuels = [module[0] for module in cursor.fetchall()]
        except:
            cx_pool.close(cursor)
            # Erreur de lecture des modules
            print "\nErreur de lecture de la liste des modules\n"
            return 0, u("Erreur de lecture de la liste des modules")
        cx_pool.close(cursor)
        new_modules = []
        for version, modules in config.liste_modules.items():
            for module in modules:
                if module not in modules_actuels:
                    # ajout d'un nouveau module
                    try:
                        new_modules.append(module)
                        self.xmlrpc_add_module("", module, version)
                    except:
                        return 0, u("erreur d'ajout du module %s" % module)
        # log de la création des modules
        if len(new_modules) > 0:
            print "\nNouveaux modules: %s\n" % ", ".join(new_modules)
            return 1, u("Nouveaux modules : %s" % ", ".join(new_modules))
        else:
            return 1, u("OK")

    def _got_modules(self,modules,cred_user):
        """Récupération de la table module depuis le backend
        """
        l=[]
        for module in modules:
            try:
                self.parent.s_pool.check_mod_credential(cred_user, module[0])
            except ResourceAuthError:
                # non autorisé : module suivant
                continue
            if module[2] == None:
                version = 1
            else:
                version = module[2]
            l.append({'id':module[0],'libelle':module[1],'version':version})
        return 1,u(l)

    ##############################
    ## gestion de la table modules
    ##############################


    def xmlrpc_add_module(self, cred_user, libelle, version=3):
        """ajoute un module dans la base de données et crée l'arborescence correspondante"""
        # on vérifie que les données obligatoires sont remplies
        if libelle:
            query = """insert into modules (libelle, version) values (E'%s',%s)""" % (adbapi.safe(libelle),int(version))
            # on effectue l'insertion (l'existence est à tester dans l'application cliente)
            return self.dbpool.runOperation(query).addCallbacks(self._add_module1,db_client_failed,callbackArgs=[libelle, version])
        else:
            # des attributs manquent
            return 0, u("""donnez un libellé""")

    def _add_module1(self,resultat,libelle,version):
        """récupère l'id du module créé"""
        # requête pour récupérer l'id
        query="""select id from modules where libelle ilike E'%s'""" % adbapi.safe(libelle)
        return self.dbpool.runQuery(query).addCallbacks(self._add_module2,db_client_failed,callbackArgs=[libelle,version])

    def _add_module2(self,row,libelle,version):
        """crée la variante standard d'un nouveau module"""
        if row ==[]:
            return 0,u('erreur de création du module dans la base')
        else:
            id_module=row[0][0]
            # on crée la variante standard du module
            query = """insert into variantes (module,libelle) values (%s, 'standard')""" % id_module
            return self.dbpool.runOperation(query).addCallbacks(self._add_module3,db_client_failed,callbackArgs=[id_module,libelle,version])
            
    def _add_module3(self,resultat,id_module,libelle,version):
        """récupère l'id de la variante standard"""
        query = """select id,module from variantes where module = %s and libelle = 'standard'""" % id_module
        return self.dbpool.runQuery(query).addCallbacks(self._add_module4,db_client_failed,callbackArgs=[libelle,version])


    def _add_module4(self,row,libelle,version):
        """met en place l'arborescence du nouveau module et de ses variantes"""
        if row == []:
            return 0,u('erreur de creation de la variante par défaut dans la base')
        else:
            id_variante = row[0][0]
            id_module = row[0][1]
            # on créé l'arborescence pour le module
            module_dir = os.path.abspath(config.PATH_MODULES)+os.sep+str(id_module)
            try:
                os.makedirs(module_dir)
                # lien du(des) dictionnaire(s)
                if int(version) == 1:
                    dico_module=os.path.abspath(config.ROOT_DIR)+'/dictionnaires/dico-%s' % libelle
                    os.system('ln -s %s %s' % (dico_module,module_dir+os.sep+'dictionnaire'))
                else:
                    # module eole 2
                    dico_module=os.path.abspath(config.ROOT_DIR)+'/dictionnaires/%s' % libelle
                    if os.path.isdir(dico_module):
                        # modules de base eole
                        os.system('ln -s %s %s' % (dico_module, os.path.join(module_dir,'dicos')))
                        # création d'un fichier de valeurs par défaut
                        d = EoleDict()
                        d.read_dir(os.path.join(module_dir,'dicos'))
                        # sauvegarde des valeurs sans vérifications pour créer le fichier
                        d.save_values(os.path.join(module_dir,'module.eol'),True)
                    else:
                        # module additionnel
                        os.makedirs(os.path.join(module_dir,'dicos'))
                # on crée la variante par défaut (module standard)
                os.makedirs(module_dir+os.sep+'variantes'+os.sep+str(id_variante)+os.sep+'dicos')
                os.makedirs(module_dir+os.sep+'variantes'+os.sep+str(id_variante)+os.sep+'patchs')
                os.makedirs(module_dir+os.sep+'variantes'+os.sep+str(id_variante)+os.sep+'fichiers_perso')
                os.makedirs(module_dir+os.sep+'variantes'+os.sep+str(id_variante)+os.sep+'fichiers_zephir')
            except:
                try:
                    shutil.rmtree(module_dir)
                except:
                    # aucun répertoire créé
                    pass
                query = """delete from variantes where module = %s""" % id_module
                self.dbpool.runOperation(query)
                query = """delete from modules where id = %s""" % id_module
                self.dbpool.runOperation(query)
                return 0, u("""erreur de creation de l'arborescence du module""")
            # on récupère les données
            # self.upload_module(id,adresse_donnees)
            return 1, u(id_module)

    def xmlrpc_del_module(self,cred_user,id_module):
        """supprime un module et ses variantes"""
        if int(id_module) >= 0:
            self.parent.s_pool.check_mod_credential(cred_user, id_module)
            # suppression des variantes dans la base de données
            query = "delete from variantes where module = %d" % int(id_module)
            return self.dbpool.runOperation(query).addCallbacks(self._del_module,db_client_failed,callbackArgs=[id_module])
        else:
            return 0, u("""donnez l'id d'un module (nombre entier)""")

    def _del_module(self,retour,id_module):
        """supprime de la base les services liés au module"""
        query = "delete from services where module = %d" % int(id_module)
        return self.dbpool.runOperation(query).addCallbacks(self._del_module1,db_client_failed,callbackArgs=[id_module])

    def _del_module1(self,retour,id_module):
        """supprime le module dans la base de données"""
        query = "delete from modules where id = %s" % id_module
        return self.dbpool.runOperation(query).addCallbacks(self._del_module2,db_client_failed,callbackArgs=[id_module])

    def _del_module2(self,retour,id_module):
        """supprime l'arborescence du module dans l'arborescence zephir"""
        module_dir=os.path.abspath(config.PATH_MODULES)+os.sep+str(id_module)
        try:
            shutil.rmtree(module_dir)
        except:
            return 1,u("""erreur de supression du répertoire du module""")
        return 1,u('ok')

    def xmlrpc_edit_module(self,cred_user,id_module,dico_modifs):
        """modification d'un module 
        cette fonction prend en compte un dictionnaire qui indique les 
        champs à modifier et leur nouvelle valeur. l'application cliente
        doit s'assurer que ces champs existent dans la base"""
        self.parent.s_pool.check_mod_credential(cred_user, id_module)
        if dico_modifs == {}:
            return 1,u("""aucune modification demandée""")
        # on vérifie que l'identifiant n'est pas modifié
        if 'id' in dico_modifs.keys():
            return 0,u("""l'identifiant ne peut pas être modifié""")
        # construction de la requête SQL de modification
        requete=["update modules set "]
        for cle in dico_modifs.keys():
            requete.append(str(cle))
            requete.append("=E'")
            requete.append(adbapi.safe(str(dico_modifs[cle])))
            requete.append("', ")
        string_fin=""" where id='%s'""" % id_module
        query="".join(requete)[:-2]
        query += string_fin
        return self.dbpool.runOperation(query).addCallbacks(lambda x:(1,'ok'), db_client_failed)

    def xmlrpc_get_module(self,cred_user, id_module=None):
        """récupère un module précis dans la base ou tous les modules"""
        if id_module :
            # on récupère le module demandé
            query = """select id, libelle, version from modules where id = %s""" % id_module
            return self.dbpool.runQuery(query).addCallbacks(self._got_modules,db_client_failed,callbackArgs=[cred_user])
        else :
            # sinon on renvoie tous les modules
            query = """select id, libelle, version from modules order by version desc, libelle"""
            return self.dbpool.runQuery(query).addCallbacks(self._got_modules,db_client_failed,callbackArgs=[cred_user])

    def xmlrpc_get_dico(self,cred_user,id_module,id_variante=None):
        """récupération du dictionnaire d'un module (valeurs par défaut)"""
        # recherche du répertoire ou le dictionnaire est stocké
        try:
            query = "select version from modules where id = %d" % int(id_module)
        except:
            return 0, u("""donnez l'id d'un module (nombre entier)""")
        return self.dbpool.runQuery(query).addCallbacks(self._get_dico,db_client_failed,callbackArgs=[id_module, id_variante])

    def _get_dico(self,data,id_module,id_variante=None):
        # détection version de créole
        try:
            version = int(data[0][0])
        except:
            version = 3

        # dictionnaire de base du module
        if version == 1:
            # module Eole1
            path_dico = os.path.abspath(config.PATH_ZEPHIR)+os.sep+'modules'+os.sep+str(id_module)+os.sep+'dictionnaire'
            # envoi des fichiers de configuration au programme de saisie
            try:
                fic = open(path_dico,'r')
            except:
                return 0, u("""dictionnaire non présent""")

            # dictionnaire du module
            lines = fic.readlines()
            fic.close()
            dictionnaire = [ unicode(line, 'ISO-8859-1').encode('UTF-8') for line in lines ]
            # dictionnaires additionnels
            if id_variante is not None:
                dicos=os.path.abspath(config.PATH_ZEPHIR)+os.sep+'modules'+os.sep+str(id_module)+os.sep+'variantes'+os.sep+str(id_variante)+os.sep+'dicos'
                for nom_fic in os.listdir(dicos):
                    if nom_fic.endswith('.eol') or nom_fic.endswith('.xml'):
                        # ajout d'un dictionnaire
                        fic = open(dicos+os.sep+nom_fic,'r')
                        lines = fic.readlines()
                        fic.close()
                        dictionnaire.extend([ unicode(line, 'ISO-8859-1').encode('UTF-8') for line in lines ])
                        
            # dictionnaire complet (dictionnaire module + dico locaux)
            dic_complet=dico.DicoEole(dico_zephir=[dictionnaire])
                    
            # si un dico.eol existait déjà, on reprend les anciennes valeurs
            if id_variante is not None:
                path_dicovar = os.path.abspath(config.PATH_ZEPHIR)+os.sep+'modules'+os.sep+str(id_module)+os.sep+'variantes'+os.sep+str(id_variante)+os.sep+'dico.eol'
                if os.path.isfile(path_dicovar):
                    # on instancie un dictionnaire depuis dico.eol
                    # et un autre avec le dictionnaire complet
                    f=file(path_dicovar)
                    lines = f.readlines()
                    f.close()
                    data = [ unicode(line, 'ISO-8859-1').encode('UTF-8') for line in lines ]
                    # dictionnaire des valeurs actuelles
                    dic_actuel=dico.DicoEole(dico_zephir=[data]).ParseDico()
                    dico_final = {}
                    # on réinsère les données existantes dans le dictionnaire
                    for variable, data in dic_complet.dictionnaire.items():
                        # si la variable existait déjà, on reprend sa valeur
                        if dic_actuel.has_key(data[0]):
                            data[1] = dic_actuel[data[0]][0]
                        dico_final[variable]=data
                    # on remet le dictionnaire modifié en place dans dico_serveur
                    dic_complet.dictionnaire = dico_final
                    
            dicos=[base64.encodestring(dic_complet.save("/dev/null"))]
        else:
            # version creole2
            # chargement de configuration pour un serveur Eole2
            path_module = os.path.abspath(config.PATH_ZEPHIR)+os.sep+'modules'+os.sep+str(id_module)
            path_dico = path_module+os.sep+'dicos'
            # définition des dictionnaires à charger
            dict_dirs = [path_dico]
            if id_variante is not None:
                dict_dirs.append(os.path.join(path_module,'variantes',str(id_variante),'dicos'))
            # création de l'objet ZephirDict et lecture des valeurs existantes
            dict_zeph = ZephirDict(dicos=dict_dirs, confdir=path_dico, mode='config', version='creole2')
            if os.path.isfile(os.path.join(path_module,'module.eol')):
                dict_zeph.dico.load_values(os.path.join(path_module,'module.eol'))
            # traitement de la variable si demandé
            if id_variante is not None:
                path_var = os.path.join(path_module,'variantes',str(id_variante),'dico.eol')
                dict_zeph.dico.load_values(os.path.join(path_var))
            dicos = dict_zeph.get_dict()
            if dicos == None:
                dicos = []

        return 1,u(dicos)
 
    def xmlrpc_get_vars(self,cred_user,id_module):
        """récupération des libellés/familles des variables eole pour un module"""
        # recherche du répertoire ou le dictionnaire est stocké
        try:
            query = "select id,version from modules where id = %d" % int(id_module)
        except:
            return 0, u("""donnez l'id d'un module (nombre entier)""")
        return self.dbpool.runQuery(query).addCallbacks(self._get_vars,db_client_failed)

    def _get_vars(self,data):
        # détection version de créole
        try:
            id_module = int(data[0][0])
        except:
            return 0, u("""module non trouvé""")
        try:
            version = int(data[0][1])
        except:
            version = 3
        # dictionnaire de base du module
        if version == 1:
            # module Eole1
            creole_files = [os.path.abspath(config.PATH_ZEPHIR)+os.sep+'modules'+os.sep+str(id_module)+os.sep+'dictionnaire']
            # récupération des familles de variables
            dicos = []
            for dic in creole_files:
                try:
                    # lecture du contenu des dictionnaires
                    fic = open(dic,'r')
                    lines = fic.readlines()
                    fic.close()
                    data = [ unicode(line, 'ISO-8859-1').encode('UTF-8') for line in lines ]
                    # stockage du contenu du dictionnaire
                    dicos.append(data)
                except OSError:
                    pass
            dict_zeph = ZephirDict(dicos=dicos, confdir='', mode='dico', version='creole1')
            families = {}
        else:
            # version creole2
            # chargement de configuration pour un serveur Eole2
            path_module = os.path.abspath(config.PATH_ZEPHIR)+os.sep+'modules'+os.sep+str(id_module)
            path_dico = path_module+os.sep+'dicos'
            # définition des dictionnaires à charger
            dict_dirs = [path_dico]
            # création de l'objet ZephirDict et lecture des valeurs existantes
            dict_zeph = ZephirDict(dicos=dict_dirs, confdir=path_dico, mode='config', version='creole2')
            families = {}
            menu = dict_zeph.get_menu(True)
            for family in menu:
                for var in family[2]:
                    families[var] = family[0]

        # dictionnaire variable, libelle
        vars_dict = {}
        try:
            var_data = dict_zeph.get_first()
            while 1:
                if families.has_key(var_data[0]):
                    family = families[var_data[0]]
                else:
                    family = ''
                vars_dict[var_data[0]] = [var_data[2],family]
                var_data = dict_zeph.get_next()
        except:
            # on arrive en fin de dictionnaire
            pass

        return 1,u(vars_dict)

    def xmlrpc_save_dico(self,cred_user,dico_b64,id_module,id_variante=None,pass_var=""):
        """mise à jour du dictionnaire d'un module (valeurs par défaut)"""
        # recherche du libellé du module
        self.parent.s_pool.check_mod_credential(cred_user, id_module)
        if id_variante is not None:
            self.parent.s_pool.check_var_credential(cred_user, id_variante)
            query = "select modules.id, modules.libelle, modules.version, variantes.owner, variantes.passmd5 \
                    from modules,variantes where modules.id=%s and modules.id=variantes.module and variantes.id=%s" \
                    % (id_module, id_variante)
        else:
            query = """select id, libelle, version from modules where id=%s""" % id_module
        return self.dbpool.runQuery(query).addCallbacks(self._save_dico, db_client_failed, callbackArgs=[cred_user,dico_b64,id_variante,pass_var])

    def _save_dico(self,data,cred_user,dico_b64,id_variante=None,pass_var=""):
        if data == []:
            return 0,u("""module non retrouvé""")
        # récupération id et libellé module
        id_module = data[0][0]
        libelle = data[0][1]
        try:
            version = int(data[0][2])
        except:
            version = 3
        if id_variante is not None:
            # vérification owner/mot de passe
            owner = data[0][3]
            passmd5 = data[0][4]
            if owner != cred_user:
                # vérification du mot de passe
                if passmd5 != pass_var and passmd5 not in [None,'']:
                    # mauvais mot de passe
                    return 0,u('mot de passe incorrect pour cette variante')
        # gestion des différentes versions de creole
        if version == 1:
            try:
                # décodage du dictionnaire
                dico = base64.decodestring(dico_b64)
                # creole 1 -> encodage en latin-1
                dico = unicode(dico,'UTF-8').encode('ISO-8859-1')
                # sauvegarde
                if id_variante is None:
                    path_dico = os.path.abspath(config.ROOT_DIR)+'/dictionnaires/dico-%s' % libelle
                else:
                    path_dico = os.path.abspath(config.PATH_ZEPHIR)+os.sep+'modules'+os.sep+str(id_module)+os.sep+'variantes'+os.sep+str(id_variante)+os.sep+'dico.eol'
                fic_zephir = open(path_dico,'w')
                fic_zephir.write(dico)
                fic_zephir.close()
            except:
                return 0,u("""erreur de sauvegarde du dictionnaire""")
        else:
            if id_variante is None:
                path_dico = os.path.abspath(config.PATH_ZEPHIR)+os.sep+'modules'+os.sep+str(id_module)+os.sep+'module.eol'
            else:
                path_dico = os.path.abspath(config.PATH_ZEPHIR)+os.sep+'modules'+os.sep+str(id_module)+os.sep+'variantes'+os.sep+str(id_variante)+os.sep+'dico.eol'
            values = eval(dico_b64[-1])
            try:
                parser = ConfigParser.ConfigParser()
                parser._sections = values
                fic_zephir = open(path_dico,'w')
                parser.write(fic_zephir)
                fic_zephir.close()
            except:
                return 0,u("""erreur de sauvegarde du dictionnaire""")

        return 1,u('ok')

    def xmlrpc_get_mod_dict(self,cred_user,id_module):
        """renvoie la liste des dictionnaires d'un module.
        utile pour les modules eole2 (fichier fixé pour les modules eole1)
        """
        self.parent.s_pool.check_mod_credential(cred_user, id_module)
        query = "select id, version from modules where id=%s" % int(id_module)
        return self.dbpool.runQuery(query).addCallbacks(self._get_mod_dict, db_client_failed)

    def _get_mod_dict(self,data):
        # définition du chemin vers les dictionnaires
        id_module, version = data[0]
        try:
            version = int(version)
        except:
            version = 3
        if version == 1:
            # non géré sur creole1
            return 1, []
        try:
            dest_dir=os.path.join(os.path.abspath(config.PATH_MODULES),str(id_module),"dicos")
            content = []
            # cas d'un répertoire
            for dict in os.listdir(dest_dir):
                if dict.endswith('.xml'):
                    content.append(dict)
            return 1, u(content)
        except:
            return 0,u("""erreur de parcours du répertoire des dictionnaires""")

    def xmlrpc_del_mod_dict(self,cred_user,id_module,dico_name):
        """renvoie la liste des dictionnaires d'un module.
        utile pour les modules eole2 (fichier fixé pour les modules eole1)
        """
        # définition du chemin vers les dictionnaires
        self.parent.s_pool.check_mod_credential(cred_user, id_module)
        try:
            fic_dest = os.path.join(os.path.abspath(config.PATH_MODULES),str(id_module),"dicos",dico_name)
            assert os.path.isfile(fic_dest)
        except:
            return 0,u("""dictionnaire inexistant %s""" % dico_name)
        # on supprime le fichier
        try:
            os.unlink(fic_dest)
            return 1, "OK"
        except:
            return 0,u("""erreur lors de la suppression du dictionnaire""")

    ####################################
    ## gestion des variantes des modules
    ####################################

    def _got_variantes(self,variantes,cred_user):
        """formate les données lues dans la table variantes pour
        l'envoyer à une application zephir (liste de dictionnaires)"""
        l=[]
        for variante in variantes:
            owner = variante[3]
            if owner == None:
                owner = ""
            try:
                self.parent.s_pool.check_var_credential(cred_user, variante[0])
            except ResourceAuthError:
                # non autorisé : module suivant
                continue
            l.append({'id':variante[0],'module':variante[1],'libelle':variante[2],'owner':owner})
        return 1,u(l)

    def xmlrpc_copy_variante(self,cred_user,module,id_variante,libelle,passmd5,keep_perms=False):
        """Copie une variante existante sur une nouvelle variante"""
        # recherche des données de l'ancienne variante
        try:
            var_src = int(id_variante)
            module = int(module)
            assert libelle != ""
        except:
            return 0, u("""paramètres invalides""")

        # lecture du mot de passe et du module de l'ancienne variante
        query = """select module,owner,passmd5 from variantes where id=%s""" % (var_src)
        return self.dbpool.runQuery(query).addCallbacks(self._copy_variante, db_client_failed,callbackArgs=[cred_user,var_src,libelle,module,passmd5,keep_perms])
            
    def _copy_variante(self,data,cred_user,var_src,libelle,module,passmd5,keep_perms):
        """insertion de la nouvelle variante"""
        try:
            module_src = data[0][0]
            if keep_perms:
                # on veut conserver le propriétaire/mot de passe
                cred_user = data[0][1]
                passmd5 = data[0][2]
        except:
            # variante créée non trouvée dans la base
            return 0,u("""module d'origine non retrouvé dans la base""")

        query = """insert into variantes (module,libelle,owner,passmd5) values (%s,E'%s','%s','%s')""" % (module,adbapi.safe(libelle),cred_user,passmd5)
        # id assigné automatiquement dans postgresql
        # on effectue l'insertion
        return self.dbpool.runOperation(query).addCallbacks(self._copy_variante2, db_client_failed,callbackArgs=[var_src,module_src,libelle,module])


    def _copy_variante2(self,data,var_src,module_src,libelle,module):
        """recherche de l'id de la variante créée"""

        query = """select * from variantes where libelle=E'%s' and module=%s""" % (adbapi.safe(libelle), module)
        return self.dbpool.runQuery(query).addCallbacks(self._copy_variante3, db_client_failed,callbackArgs=[var_src,module_src])

    def _copy_variante3(self,data,var_src,module_src):
        """copie l'arborescence de la variante source"""
        try:
            var_dest = data[0][0]
            module = data[0][1]
            libelle = data[0][2]
        except:
            # variante créée non trouvée dans la base
            return 0,u("""nouvelle variante non retrouvée dans la base""")
        else:
            # on recopie l'arborescence de la variante source
            rep_src=os.path.abspath(config.PATH_MODULES)+os.sep+str(module_src)+os.sep+'variantes'+os.sep+str(var_src)
            rep_dest=os.path.abspath(config.PATH_MODULES)+os.sep+str(module)+os.sep+'variantes'+os.sep+str(var_dest)
            try:
                shutil.copytree(rep_src,rep_dest)
            except:
                traceback.print_exc()
                return 0,u("""erreur de copie de l'arborescence de la variante""")

        query = """select id, libelle, version from modules where id = %s or id = %s""" % (module_src, module)
        return self.dbpool.runQuery(query).addCallbacks(self._copy_variante4, db_client_failed,callbackArgs=[var_src, var_dest, module_src, module])

    def _copy_variante4(self, data, var_src, var_dest, module_src, module):
        """stocke la concordance entre les variantes si nécessaire"""
        version_src = version_dest = 0
        libelle_src = libelle_dest = ""
        try:
            for mod in data:
                if mod[0] == module_src:
                    version_src = int(mod[2])
                    libelle_src = mod[1][:mod[1].rindex("-")]
                elif mod[0] == module:
                    version_dest = int(mod[2])
                    libelle_dest = mod[1][:mod[1].rindex("-")]
        except:
            print 'copie de variante : erreur lors de la recherche des version des modules sources et destination'
        # stockage de la liaison entre les 2 variantes si copie pour upgrade entre 2 version d'un module
        print version_src, version_dest, libelle_src, libelle_dest
        version_delta = version_dest - version_src
        if libelle_src == libelle_dest and version_delta == 1:
            query = """insert into migration_variantes (id_source, id_dest) values (%s,%s)""" % (var_src, var_dest)
            return self.dbpool.runOperation(query).addCallbacks(lambda x:(1, var_dest), db_client_failed)
        else:
            return 1, var_dest

    def xmlrpc_add_variante(self,cred_user, module, libelle, pass_var=""):
        """ajoute une variante à un module"""
        self.parent.s_pool.check_mod_credential(cred_user, module)
        # on vérifie qu'on a bien récupéré un module
        if module and libelle:
            query = """insert into variantes (module,libelle,owner,passmd5) values (%s,E'%s','%s','%s')""" % (module,adbapi.safe(libelle),cred_user,pass_var)
            # id assigné automatiquement dans postgresql
            # on effectue l'insertion
            return self.dbpool.runOperation(query).addCallbacks(self._add_variante1, db_client_failed,callbackArgs=[libelle,module])
        else:
            # des attributs manquent
            return 0, u("""donnez un id de module et un libellé""")
        
    def _add_variante1(self,data,libelle,module):
        """recherche de l'id de la variante créée"""
        # on récupère l'id de la variante créée
        query = """select * from variantes where libelle ilike E'%s' and module = %s""" % (adbapi.safe(libelle), module)
        return self.dbpool.runQuery(query).addCallbacks(self._add_variante2,db_client_failed)
            
    def _add_variante2(self,variante):
        """met en place l'arborescence d'une nouvelle variante"""
        try:
            id_variante = variante[0][0]
            module = variante[0][1]
            libelle = variante[0][2]
        except:
            # variante créée non trouvée dans la base
            return 0,u("""nouvelle variante non retrouvée dans la base""")
        else:
            # on créé l'arborescence de la variante dans les modules
            os.makedirs(os.path.abspath(config.PATH_MODULES)+os.sep+str(module)+os.sep+'variantes'+os.sep+str(id_variante))
            os.makedirs(os.path.abspath(config.PATH_MODULES)+os.sep+str(module)+os.sep+'variantes'+os.sep+str(id_variante)+os.sep+'patchs')
            os.makedirs(os.path.abspath(config.PATH_MODULES)+os.sep+str(module)+os.sep+'variantes'+os.sep+str(id_variante)+os.sep+'dicos')
            os.makedirs(os.path.abspath(config.PATH_MODULES)+os.sep+str(module)+os.sep+'variantes'+os.sep+str(id_variante)+os.sep+'fichiers_perso')
            os.makedirs(os.path.abspath(config.PATH_MODULES)+os.sep+str(module)+os.sep+'variantes'+os.sep+str(id_variante)+os.sep+'fichiers_zephir')
            return 1,id_variante


    def xmlrpc_del_variante(self,cred_user,id_variante):
        """suppression d'une variante de la base zephir"""
        self.parent.s_pool.check_var_credential(cred_user, id_variante)
        if id_variante:
            query = """select * from variantes where id = %s""" % id_variante
            # on récupère les données de la variante
            return self.dbpool.runQuery(query).addCallbacks(self._del_variante, db_client_failed)
        else:
            return 0,u("""donnez un identifiant de variante""")

    def _del_variante(self,variante):
        """supprime la variante de la base de données"""
        try:
            id_variante = variante[0][0]
            id_module = variante[0][1]
            libelle = variante[0][2]
        except:
            # variante non trouvée dans la base
            return 0,u("""variante non trouvée dans la base""")
        else:
            # on interdit la supression de la variante standard
            if libelle == 'standard':
                return 0, u("""la variante standard ne peut pas être supprimée""")
            # suppression variante dans la base
            query = """delete from variantes where id = %s""" % id_variante
            return self.dbpool.runOperation(query).addCallbacks(self._del_variante2, db_client_failed,callbackArgs=[id_module,id_variante])

    def _del_variante2(self,result,id_module,id_variante):
        """supression de l'arborescence de la variante"""
        # ok, on supprime son arborescence dans zephir/modules/variantes/id_variante
        variante_dir=os.path.abspath(config.PATH_MODULES)+os.sep+str(id_module)+os.sep+"variantes"+os.sep+str(id_variante)
        try:
            shutil.rmtree(variante_dir)
        except:
            # remonter code -1 si supression de répertoire impossible ?
            return 0,u("""erreur de supression du répertoire de la variante""")
        return 1, u('ok')
   
    def xmlrpc_fichiers_variante(self,cred_user,id_variante):
        """ retourne la liste des fichiers personnalisés pour cette variante """
        self.parent.s_pool.check_var_credential(cred_user, id_variante)
        query = """select id,module from variantes where id = %s""" % id_variante
        return self.dbpool.runQuery(query).addCallbacks(self._fichiers_zephir,db_client_failed)

    def _fichiers_zephir(self,data):
        """recherche les fichiers liés à une variante"""
        if data == []:
            return 0,u("serveur inconnu de zephir")
        else:
            id_variante = int(data[0][0])
            id_module = int(data[0][1])
            # répertoire de stockage de la variante sur zephir
            variante_dir = os.path.abspath(config.PATH_ZEPHIR)+os.sep+'modules'+os.sep+str(id_module)+os.sep+'variantes'+os.sep+str(id_variante)
            # création du dictionnaire de listes des différents fichiers
            dico_res={}
            try:
                # dictionnaires additionnels
                liste_dicos = []
                liste_dicos_var = []
                for fic in os.listdir(variante_dir+os.sep+'dicos'):
                    if fic.endswith('.eol') or fic.endswith('.xml'):
                        liste_dicos_var.append(fic)
                dico_res['dicos_var'] = liste_dicos_var
            except OSError:
                dico_res['dicos_var'] = ['répertoire non trouvé !']
            try:    
                # fichiers templates variante
                dico_res['persos_var'] = (os.listdir(variante_dir+os.sep+'fichiers_perso'))
            except OSError:
                dico_res['persos_var'] = ['répertoire non trouvé !']
            try:
                # patchs variante
                dico_res['patchs_var'] = os.listdir(variante_dir+os.sep+'patchs')
            except OSError:
                dico_res['patchs_var'] = ['répertoire non trouvé !']
            try:
                # RPMS variante
                # lecture de la liste des rpms supplémentaires
                fic = open(variante_dir+'/fichiers_zephir/fichiers_variante')
                data = fic.read().split("\n")
                fic.close()
                # recherche de la section des RPMS
                liste_pkg_var = []
                section_rpm = 0
                for ligne in data:
                    ligne = ligne.strip()
                    if section_rpm == 1:
                        # on regarde si on a bien affaire à un paquetage
                        if not ligne.startswith('#') and ligne != '':
                            # on affiche le nom du paquetage
                            liste_pkg_var.append(ligne)
                    if ligne == '%%':
                        section_rpm = 1
                dico_res['rpms_var'] = liste_pkg_var
            except IOError:
                dico_res['rpms_var'] = []
            try:
                # fichiers du module
                liste_fic=[]
                try:
                    f=open(variante_dir+os.sep+'fichiers_zephir/fichiers_variante')
                    old_content=f.read()
                    f.close()
                    fichiers=old_content.split('%%\n')[0]
                except:
                    fichiers=""
                for f in fichiers.split('\n'):
                    if f.strip().startswith("""/"""):
                        liste_fic.append(f)
                dico_res['fichiers_var'] = liste_fic
#                for fic in os.listdir(variante_dir+os.sep+'fichiers_zephir'):
#                    if fic != 'fichiers_variante':
#                        liste_dicos.append(fic)
#                dico_res['fichiers_var'] = liste_dicos
            except:
                dico_res['fichiers_var'] = ['répertoire non trouvé !']
            return 1,u(dico_res)

    def xmlrpc_get_variante_perms(self, cred_user, id_variante, filepath=""):
        """renvoie les informations de permissions associées à un(des) fichier(s)
        """
        # on recherche le répertoire ou la variante est stockée
        query = """select id,module from variantes where id=%s""" % id_variante
        return self.dbpool.runQuery(query).addCallbacks(self._get_variante_perms,db_client_failed,callbackArgs=[filepath])

    def _get_variante_perms(self, data, filepath):
        """crée l'archive de la variante et la renvoie"""
        if data != []:
            id_variante = data[0][0]
            id_module = data[0][1]
            # répertoire de la variante
            var_dir=os.path.abspath(config.PATH_MODULES)+os.sep+str(id_module)+os.sep+"variantes"+os.sep+str(id_variante)
        try:
            id_var = int(id_variante)

        except KeyError, ValueError:
            return 0, u("""variante inconnue : %s""" % str(id_var))
        else:
            result = self.parent.s_pool.get_file_perms(var_dir, filepath)
            return 1, u(result)

    def xmlrpc_del_variante_perms(self, cred_user, id_variante, filepath="", pass_var=""):
        """supprime les informations de permissions associées à un(des) fichier(s)
        """
        # on recherche le répertoire ou la variante est stockée
        query = """select id,module,owner,passmd5 from variantes where id=%s""" % id_variante
        return self.dbpool.runQuery(query).addCallbacks(self._del_variante_perms,db_client_failed,callbackArgs=[cred_user,filepath,pass_var])

    def _del_variante_perms(self, data, cred_user, filepath, pass_var):
        """crée l'archive de la variante et la renvoie"""
        if data != []:
            id_variante = data[0][0]
            id_module = data[0][1]
            owner = data[0][2]
            passmd5 = data[0][3]
            # vérification owner/mot de passe
            if owner != cred_user:
                # vérification du mot de passe
                if passmd5 != pass_var and passmd5 not in [None,'']:
                    # mauvais mot de passe
                    return 0,u('mot de passe incorrect pour cette variante')
            # répertoire de la variante
            var_dir=os.path.abspath(config.PATH_MODULES)+os.sep+str(id_module)+os.sep+"variantes"+os.sep+str(id_variante)
        try:
            id_var = int(id_variante)
        except KeyError, ValueError:
            return 0, u("""variante inconnue : %s""" % str(id_var))
        else:
            result = self.parent.s_pool.del_file_perms(var_dir, filepath)
            return 1, u(result)

    def xmlrpc_set_variante_perms(self, cred_user, id_variante, rights, pass_var=""):
        """enregistre les informations de permissions associées à un(des) fichier(s)
        @param rights: dictionnaire au format suviant : {'filepath':[mode,ownership]}
        """
        # on recherche le répertoire ou la variante est stockée
        query = """select id,module,owner,passmd5 from variantes where id=%s""" % id_variante
        return self.dbpool.runQuery(query).addCallbacks(self._set_variante_perms,db_client_failed,callbackArgs=[cred_user,rights,pass_var])

    def _set_variante_perms(self, data, cred_user, rights, pass_var):
        """crée l'archive de la variante et la renvoie"""
        if data != []:
            id_variante = data[0][0]
            id_module = data[0][1]
            owner = data[0][2]
            passmd5 = data[0][3]
            # vérification owner/mot de passe
            if owner != cred_user:
                # vérification du mot de passe
                if passmd5 != pass_var and passmd5 not in [None,'']:
                    # mauvais mot de passe
                    return 0,u('mot de passe incorrect pour cette variante')
            # répertoire de la variante
            var_dir=os.path.abspath(config.PATH_MODULES)+os.sep+str(id_module)+os.sep+"variantes"+os.sep+str(id_variante)
        try:
            id_var = int(id_variante)
        except KeyError, ValueError:
            return 0, u("""variante inconnue : %s""" % str(id_var))
        else:
            res = self.parent.s_pool.set_file_perms(rights, var_dir)
            if res:
                return 1,"OK"
            else:
                return 0, u("""erreur d'enregistrement des permissions""")

    def xmlrpc_get_variante(self,cred_user,id_variante=None):
        """récupère la liste d'une variante (ou toutes)"""
        if id_variante :
            query = """select * from variantes where id = %s """ % id_variante
            return self.dbpool.runQuery(query).addCallbacks(self._got_variantes,db_client_failed,callbackArgs=[cred_user])
        else :
            query = """select * from variantes order by module desc,  id desc"""
            return self.dbpool.runQuery(query).addCallbacks(self._got_variantes,db_client_failed,callbackArgs=[cred_user])

    def xmlrpc_edit_variante(self,cred_user,id_variante,dico_modifs,pass_var=""):
        """modification d'une variante
        cette fonction prend en compte un dictionnaire qui indique les 
        champs à modifier et leur nouvelle valeur. l'application cliente
        doit s'assurer que ces champs existent dans la base"""
        self.parent.s_pool.check_var_credential(cred_user, id_variante)
        # vérification du propriétaire/mot de passe
        query = """select owner,passmd5 from variantes where id=%s""" % id_variante
        return self.dbpool.runQuery(query).addCallbacks(self._edit_variante,db_client_failed,
                                                        callbackArgs=[cred_user,id_variante,dico_modifs,pass_var])

    def _edit_variante(self,data,cred_user,id_variante,dico_modifs,pass_var):
        # on vérifie que l'identifiant n'est pas modifié
        if dico_modifs == {}:
            return 1,u("""aucune modification demandée""")
        owner = data[0][2]
        passmd5 = data[0][3]
        # vérification owner/mot de passe
        if owner != cred_user:
            # vérification du mot de passe
            if passmd5 != pass_var and passmd5 not in [None,'']:
                # mauvais mot de passe
                return 0,u('mot de passe incorrect pour cette variante')
        if ('id' in dico_modifs.keys()) or ('module' in dico_modifs.keys()):
            return 0,u("""l'identifiant et le module ne peuvent pas être modifiés""")
        if ('owner' in dico_modifs.keys()) or ('passmd5' in dico_modifs.keys()):
            return 0,u("""modification du propriétaire ou mot de passe interdits""")
        # construction de la requête SQL de modification
        requete=["update variantes set "]
        for cle in dico_modifs.keys():
            requete.append(str(cle))
            requete.append("=E'")
            requete.append(adbapi.safe(str(dico_modifs[cle])))
            requete.append("', ")
        string_fin=""" where id='%s'""" % id_variante
        query="".join(requete)[:-2]
        query += string_fin
        return self.dbpool.runOperation(query).addCallbacks(lambda x:(1,'ok'), db_client_failed)

    def xmlrpc_add_files(self,cred_user,id_variante,dico_files,passwd="",encode=False):
        """ajoute des fichiers, patchs, dictionnaires à une variante
        """
        self.parent.s_pool.check_var_credential(cred_user, id_variante)
        query="select modules.version,variantes.id,variantes.module,variantes.owner,variantes.passmd5 from variantes,modules where variantes.id=%s and modules.id=variantes.module" % id_variante
        return self.dbpool.runQuery(query).addCallbacks(self._add_files,db_client_failed,callbackArgs=[dico_files,cred_user,passwd,encode])

    def _add_files(self,data,dico_files,cred_user,passwd,encode):
        """ajoute des fichiers, patchs, dictionnaires,rpms à une variante
        """
        mod_version = data[0][0]
        try:
            mod_version = int(mod_version)
        except:
            mod_version = 3
        id_variante = data[0][1]
        id_module = data[0][2]
        owner = data[0][3]
        pass_var = data[0][4]
        # si la variante ne nous appartient pas, on interdit la modification si mauvais mot de passe
        if owner != cred_user:
            # vérification du mot de passe
            if passwd != pass_var and pass_var not in [None,'']:
                # mauvais mot de passe
                return 0,u('mot de passe incorrect pour cette variante')
        # on met en place les différents types de fichiers
        dest_dir=os.path.abspath(config.PATH_MODULES)+os.sep+str(id_module)+os.sep+"variantes"+os.sep+str(id_variante)
        # dans le cas d'un serveur eole1, on encode le contenu des fichiers en iso8859 si nécessaire
        if mod_version == 1 and encode == True:
            for type_f, files in dico_files.items():
                if type_f in ['dicos_var','patchs_var','persos_var','fichiers_var']:
                    encoded_files = []
                    for fichier in dico_files[type_f]:
                        content = unicode(base64.decodestring(fichier[1]),config.charset).encode('ISO-8859-1')
                        localpath = ""
                        if len(fichier) == 3:
                            localpath = fichier[2]
                        encoded_files.append([fichier[0], base64.encodestring(content),localpath])
                    dico_files[type_f] = encoded_files
        # dictionnaires locaux
        for dico in dico_files['dicos_var']:
            try:
                if dico[0] != "":
                    f=open(dest_dir+os.sep+'dicos'+os.sep+os.path.basename(dico[0].replace("\\","/")),'w')
                    f.write(base64.decodestring(dico[1]))
                    f.close()
            except:
                return 0,u("erreur de sauvegarde de %s" % dico)
        for template in dico_files['persos_var']:
            try:
                if template[0] != "":
                    f=open(dest_dir+os.sep+'fichiers_perso'+os.sep+os.path.basename(template[0].replace("\\","/")),'w')
                    f.write(base64.decodestring(template[1]))
                    f.close()
            except:
                return 0,u("erreur de sauvegarde de %s" % template)
        for patch in dico_files['patchs_var']:
            try:
                if patch[0] != "":
                    f=open(dest_dir+os.sep+'patchs'+os.sep+os.path.basename(patch[0].replace("\\","/")),'w')
                    f.write(base64.decodestring(patch[1]))
                    f.close()
            except:
                return 0,u("erreur de sauvegarde de %s" % patch)
        # on reprend la liste des fichiers existants
        try:
            f=open(dest_dir+os.sep+'fichiers_zephir/fichiers_variante')
            old_content=f.read()
            f.close()
            fichiers=old_content.split('%%\n')[0]
            rpms=old_content.split('%%\n')[1]
        except:
            fichiers="""# section 1
# liste des fichiers à sauvegarder
# (ne pas modifier sauf pour créer ou mettre à jour la variante)"""
            rpms="""# section 2
# inscrire les noms des paquetages qui seront installés à la mise à jour du serveur
# (ils doivent être présents sur le serveur de mise à jour)"""

        for fichier in dico_files['fichiers_var']:
            # on ajoute le fichier à la liste si il n'est pas déjà présent
            localpath = ""
            if len(fichier) == 3:
                localpath = fichier[2]
            # on ajoute le fichier à la liste si il n'est pas déjà présent et si il n'est pas dans un sous répertoire
            nom_fic = fichier[0].replace("\\","/")
            # on supprime les séparateurs en fin de ligne
            if fichier[0].endswith('/'):
                nom_fic = fichier[0][:-1]
            if fichier[0].endswith("\\"):
                nom_fic = fichier[0][:-2]
            if nom_fic not in fichiers.split('\n') and localpath == "":
                fichiers = fichiers.strip() + '\n' + nom_fic +'\n'
            # on écrit le contenu du fichier
            try:
                if nom_fic != "":
                    if localpath == "":
                        f=open(os.path.join(dest_dir,'fichiers_zephir',os.path.basename(nom_fic)),'w')
                    else:
                        f=open(os.path.join(dest_dir,localpath,os.path.basename(nom_fic)),'w')
                    f.write(base64.decodestring(fichier[1]))
                    f.close()
            except:
                return 0,u("erreur de sauvegarde de %s" % fichier)

        # rpms
        for rpm in dico_files['rpms_var']:
            # on ajoute le rpm si il n'est pas présent
            if rpm not in rpms.split('\n'):
                rpms = rpms.strip() + '\n' + rpm +'\n'

        f=open(dest_dir+os.sep+'fichiers_zephir/fichiers_variante','w')
        f.write(fichiers+"%%\n"+rpms)
        f.close()
        return 1,u("ok")

    def xmlrpc_del_files(self,cred_user,id_variante,dico_files,passwd=None):
        """suppression de fichiers, patchs, dictionnaires d'une variante
        """
        self.parent.s_pool.check_var_credential(cred_user, id_variante)
        query="select id,module,owner,passmd5 from variantes where id=%s" % id_variante
        return self.dbpool.runQuery(query).addCallbacks(self._del_files,db_client_failed,callbackArgs=[dico_files,cred_user,passwd])

    def _del_files(self,data,dico_files,cred_user,passwd):
        """supression de fichiers, patchs, dictionnaires,rpms d'une variante
        """
        id_variante = data[0][0]
        id_module = data[0][1]
        owner = data[0][2]
        pass_var = data[0][3]
        # si la variante ne nous appartient pas, on interdit la modification si mauvais mot de passe
        if owner != cred_user:
            # vérification du mot de passe
            if passwd != pass_var and pass_var not in [None,'']:
                # mauvais mot de passe
                return 0,u('mot de passe incorrect pour cette variante')
        # on met en place les différents types de fichiers
        dest_dir=os.path.abspath(config.PATH_MODULES)+os.sep+str(id_module)+os.sep+"variantes"+os.sep+str(id_variante)
        # dictionnaires locaux
        for dico in dico_files['dicos_var']:
            try:
                if dico != "":
                    os.unlink(dest_dir+os.sep+'dicos'+os.sep+dico)
            except:
                return 0,u("erreur de suppression de %s" % dico)
        for template in dico_files['persos_var']:
            try:
                if template != "":
                    os.unlink(dest_dir+os.sep+'fichiers_perso'+os.sep+template)
            except:
                return 0,u("erreur de supression de %s" % template)
            # on supprime les droits associés si nécessaire
            self.parent.s_pool.del_file_perms(dest_dir,'fichiers_perso'+os.sep+template)

        for patch in dico_files['patchs_var']:
            try:
                if patch != "":
                    os.unlink(dest_dir+os.sep+'patchs'+os.sep+patch)
            except:
                return 0,u("erreur de suppression de %s" % patch)
        # on reprend la liste des fichiers existants
        try:
            f=open(dest_dir+os.sep+'fichiers_zephir/fichiers_variante')
            old_content=f.read()
            f.close()
            fichiers=old_content.split('%%\n')[0]
            rpms=old_content.split('%%\n')[1]
        except:
            fichiers="""# section 1
# liste des fichiers à sauvegarder pour la variante
# (ne pas modifier sauf pour créer ou mettre à jour la variante)"""
            rpms="""# section 2
# inscrire les noms des paquetages qui seront installés à la mise à jour du serveur
# (ils doivent être présents sur le serveur de mise à jour)"""

        for fichier in dico_files['fichiers_var']:
            # on supprime le fichier de la liste
            liste=fichiers.split('\n')
            if fichier in liste:
                liste.remove(fichier)
                fichiers = "\n".join(liste)
                fic_path = os.path.join(dest_dir,'fichiers_zephir',os.path.basename(fichier.replace("\\","/")))
            else:
                fic_path = os.path.join(dest_dir,fichier)
            # on efface le fichier
            try:
                if fichier != "":
                    if os.path.isdir(fic_path):
                        shutil.rmtree(fic_path)
                    else:
                        os.unlink(fic_path)
            except:
                return 0,u("erreur de suppression de %s" % fichier)
            if fichier.startswith('/'):
                fic_sup = 'fichiers_zephir/'+os.path.basename(fichier.replace("\\","/"))
            else:
                fic_sup = fichier
            self.parent.s_pool.del_file_perms(dest_dir,fic_sup,True)

        # rpms
        for rpm in dico_files['rpms_var']:
            # on supprime le rpm si il existe
            liste=rpms.split('\n')
            if rpm in liste:
                liste.remove(rpm)
                rpms = "\n".join(liste)
            else:
                return 0,u("rpm non trouvé dans la liste : %s" % rpm)

        f=open(dest_dir+os.sep+'fichiers_zephir/fichiers_variante','w')
        f.write(fichiers+"%%\n"+rpms)
        f.close()
        return 1,u("ok")

    def xmlrpc_get_var_file(self,cred_user,id_var,id_module,path):
        """renvoie le contenu d'un fichier de variante"""
        # définition du chemin de la variante
        self.parent.s_pool.check_var_credential(cred_user, id_var)
        query="select variantes.id,variantes.module,modules.version from variantes,modules where variantes.id=%s and modules.id=variantes.module" % id_var
        return self.dbpool.runQuery(query).addCallbacks(self._get_var_file,db_client_failed,callbackArgs=[path])

    def _get_var_file(self,data,path):
        id_var = data[0][0]
        id_module = data[0][1]
        mod_version = data[0][2]
        try:
            mod_version = int(mod_version)
        except:
            mod_version = 3
        try:
            dest_dir=os.path.abspath(config.PATH_MODULES)+os.sep+str(id_module)+os.sep+"variantes"+os.sep+str(id_var)
        except:
            return 0,u("""lecture du fichier: paramètres non valides""")
        # on lit le fichier
        try:
            # cas d'un répertoire
            if os.path.isdir(dest_dir + os.sep + path):
                content = os.listdir(dest_dir + os.sep + path)
                return 1, u(content)
            else:
                if istextfile(dest_dir + os.sep + path):
                    f=file(dest_dir + os.sep + path)
                    content=f.read()
                    f.close()
                    # on encode le contenu en base64
                    if mod_version == 1:
                        try:
                            content = unicode(content,'ISO-8859-1').encode(config.charset)
                        except:
                            # le fichier n'est pas en unicode ??
                            print "echec d'encoding du fichier %s provenant d'un serveur eole1" % path
                    content = base64.encodestring(content)
                else:
                    content = "BINARY"
                return 1, content
        except:
            return 0,u("""erreur de lecture du fichier""")

    def xmlrpc_export_variante(self,cred_user,id_variante):
        """envoie le contenu d'une variante sur un autre zephir"""
        # on recherche le répertoire ou la variante est stockée
        query = """select id,module from variantes where id=%s""" % id_variante
        return self.dbpool.runQuery(query).addCallbacks(self._export_variante,db_client_failed)

    def _export_variante(self,data):
        """crée l'archive de la variante et le renvoie"""
        if data != []:
            id_variante = data[0][0]
            id_module = data[0][1]
            # répertoire de la variante
            parent_dir=os.path.abspath(config.PATH_MODULES)+os.sep+str(id_module)+os.sep+"variantes"
            var_dir=parent_dir+os.sep+str(id_variante)
            # on crée une archive de ce répertoire
            archive=str(time.time())
            if os.path.isdir(var_dir):
                # création du fichier tar à envoyer
                cmd_tar = ['cd ',var_dir,';','tar','-chzf','/tmp/'+archive+'.tgz','*']
                cmd_tar.append('2>&1 >/dev/null')
                # exécution de tar
                res=os.system(" ".join(cmd_tar))
                if res != 0:
                    return 0, u("""erreur de création de l'archive %s.tgz dans /tmp""" % (archive))
                else:
                    # calcul et stockage d'un checksum md5 de l'archive
                    cmd_checksum = """cd /tmp/; md5sum -b %s.tgz > %s.md5""" % (archive,archive)
                    res=os.system(cmd_checksum)
                    # on stocke en mémoire les données
                    try:
                        # le checksum
                        f = open('/tmp'+os.sep+archive+'.md5')
                        data1 = f.read()
                        f.close()
                        os.unlink('/tmp'+os.sep+archive+'.md5')
                        # l'archive
                        f = open('/tmp'+os.sep+archive+'.tgz')
                        data2 = f.read()
                        f.close()
                        os.unlink('/tmp'+os.sep+archive+'.tgz')
                        return 1,u([archive,base64.encodestring(data1),base64.encodestring(data2)])
                    except Exception, e:
                        return 0,u("""erreur lors de l'envoi de l'archive : %s""" % str(e))
            else:
                return 0, u("""répertoire %s introuvable""" % var_dir)
        else:
            return 0, u("""variante inexistante""")
                    
    def xmlrpc_import_variante(self,cred_user,pwd_var,id_local,id_distant,zephir_distant,login_distant,pwd_distant):
        """récupère le contenu d'une variante sur un autre zephir"""
        # création d'un proxy vers le serveur zephir distant
        z=xmlrpclib.ServerProxy("https://%s:%s@%s:%s" % (login_distant,pwd_distant,zephir_distant,config.PORT_ZEPHIR))
        # on vérifie l'existence des variantes des 2 cotés
        try:
            res=z.modules.get_variante(id_distant)
        except:
            return 0,u("""permissions insuffisantes""")
        if res[0] == 0:
            return 0,u("""erreur lors de la recherche de la variante d'origine""")

        # on recherche le module de la variante locale
        query = """select id, module, owner, passmd5 from variantes where id=%s""" % id_local
        return self.dbpool.runQuery(query).addCallbacks(self._import_variante,db_client_failed,callbackArgs=[z,id_distant,cred_user,pwd_var])

    def _import_variante(self,data,proxy,id_distant,cred_user,pwd_var):
        """demande l'envoi de l'archive et met en place les fichiers"""
        if data == []:
            return 0, u("""variante locale non trouvée""")
        else:
            id_variante = data[0][0]
            id_module = data[0][1]
            owner = data[0][2]
            passmd5 = data[0][3]
            # on vérifie le mot de passe de la variante
            if owner != cred_user and pwd_var != passmd5:
                return 0,u("""mauvais mot de passe de variante""")
            else:
                # définition du chemin de destination
                parent_dir=os.path.abspath(config.PATH_MODULES)+os.sep+str(id_module)+os.sep+"variantes"
                var_dir = parent_dir+os.sep+str(id_variante)
                # on regarde si des fichiers existent déjà
                if not os.path.exists(var_dir):
                    return 0,u("""répertoire de la variante de destination non trouvé""")
                # onrécupère les données de la variante source
                res=proxy.modules.export_variante(id_distant)
                if res[0]==0:
                    return 0,u("""erreur lors de la récupération de la variante""")
                else:
                    # nom de l'archive
                    archive=res[1][0]
                    # somme md5
                    var_data=base64.decodestring(res[1][1])
                    # contenu de l'archive
                    md5_data=base64.decodestring(res[1][2])
                    try:
                        # on supprime l'ancien répertoire de la variante
                        shutil.rmtree(var_dir)
                        # on recrée un répertoire vide
                        os.makedirs(var_dir)
                        # on recrée l'archive et le fichier md5
                        f=open(var_dir+os.sep+archive+'.tgz','w')
                        f.write(md5_data)
                        f.close()
                        f=open(var_dir+os.sep+archive+'.md5','w')
                        f.write(var_data)
                        f.close()
                        # on vérifie la somme md5
                        cmd_md5 = """cd %s; md5sum -c %s.md5 2>&1 > /dev/null""" % (var_dir,archive)
                        res=os.system(cmd_md5)
                        if res != 0:
                            return 0,u("""archive corrompue""")
                        # on décompresse l'archive
                        cmd_tar = """cd %s ; tar -xzf %s.tgz > /dev/null""" % (var_dir,archive)
                        res=os.system(cmd_tar)
                        if res != 0:
                            return 0,u("""erreur de décompression de l'archive""")
                        # on supprime les fichiers temporaires
                        os.unlink(var_dir+os.sep+archive+'.tgz')
                        os.unlink(var_dir+os.sep+archive+'.md5')
                    except Exception, e:
                        return 0,u("""erreur de mise en place des fichiers de la variante : %s""" % str(e))
                # l'import s'est terminé correctement
                return 1,u("ok")

