# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# prelude_rpc.py
#  
# fonctions xmlrpc pour la gestion de prélude dans Zephir
#       
###########################################################################
"""module de gestion de prelude
"""
from twisted.python import log
from twisted.internet.utils import getProcessOutput
from zephir.backend.db_utils import *
from zephir.backend import config
from zephir.backend.config import u
from zephir.backend.xmlrpceole import XMLRPCEole as XMLRPC
from zephir.backend.lib_backend import ResourceAuthError
import sys,os,shutil,time,base64,xmlrpclib

class RPCPrelude(XMLRPC):
    """serveur XMLRPC zephir pour la gestion de prélude
    """

    def __init__(self,parent):
        self.dbpool = db_connect()
        self.dbpool.noisy = 0
        XMLRPC.__init__(self)
        self.parent = parent

    def xmlrpc_register_server(self,cred_user,id_manager):
        """négocie l'enregistrement d'une sonde prelude dans prelude manager
        id_serveur : identifiant zephir du serveur sonde

        -génère un mot de passe aléatoire
        -demande le lancement de prelude-adduser sur le manager
        -renvoie le mot de passe pour permettre l'enregistrement côté sonde
        """
        # génération du mot de passe avec pwgen
        cmd_pass = getProcessOutput("/usr/bin/pwgen",
                                    args = ["-B","-s","-1"],
                                    env = {'LC_ALL': 'C'})
        return cmd_pass.addCallback(self._register_server,cred_user, id_manager)

    def _register_server(self,output,cred_user,id_manager):
        passwd = output.strip()
        # lancement de prelude-adduser sur le manager
        try:
            # lecture de la clé du manager
            serv = self.parent.s_pool.get(cred_user, int(id_manager))
            key = file(os.path.join(serv.get_confdir(),'cle_publique')).read()
            try:
                ip_manager = serv.get_config().get_value('adresse_ip_eth0')[0]
            except:
                return 0, u("""Adresse ip du manager %s non disponible""" % id_manager)
            manager_rpc = xmlrpclib.ServerProxy('https://%s:%s' % (ip_manager, config.PRELUDE_PORT))
            res = manager_rpc.register_server(xmlrpclib.base64.encodestring(key),passwd)
        except:
            import traceback
            traceback.print_exc()
            res = False
        if res == True:
            return 1, u(passwd)
        else:
            return 0, u("""Impossible de lancer prelude-adduser sur le manager""")

    def xmlrpc_get_managers(self, cred_user):
        """renvoie l'adresse du manager défini sur zephir si il existe"""
        # on récupère la liste des managers disponibles
        query = """select distinct(serveurs.id), serveurs.libelle from serveurs, modules where serveurs.module_actuel=modules.id and modules.libelle like 'preludemanager-%'"""
        return self.dbpool.runQuery(query).addCallbacks(self._get_managers, db_client_failed,callbackArgs=[cred_user])

    def _get_managers(self, data, cred_user):
        if data == []:
            # pas de serveurs preludemanager enregistrés
            return 1, {}
        else:
            managers = {}
            for manager in data:
                id_man = int(manager[0])
                libelle = manager[1]
                # récupération de l'adresse ip
                serv = self.parent.s_pool.get(cred_user, id_man)
                try:
                    ip = serv.get_config().get_value('adresse_ip_eth0')[0]
                except:
                    # configuration inconnue pour ce manager
                    ip = ''
                if ip != '':
                    managers[str(id_man)]=[ip,libelle]

        return 1, u(managers)

    def xmlrpc_register_logger(self,cred_user,id_serveur):
        """permet de vérifier si des serveurs ont été modifiés
        """
        return 1, u("""Fonction non implémentée""")

