#! /usr/bin/env python
# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# uucp_utils.py
#  
# fonctions utilitaires pour la gestion d'uucp
#       
###########################################################################

import os,time,shutil,tempfile

UUCP_DIR = "/var/spool/uucp"
CMD_UUX = "/usr/bin/uux2"
CMD_UUCP = "/usr/bin/uucp2"
LOG_FILE = "/tmp/rapport.zephir"
LOCK = "/var/spool/uucp/lock"
IGNORE_LIST = ['.ssh','.Status','.Temp','.Received','.bash_history','dead.letter']

COMMANDS = {"zephir_client save_files":"Sauvegarde de configuration",
            "zephir_client maj_auto":"Mise à jour",
            "zephir_client maj_auto E":"Mise à jour complète",
            "zephir_client maj_client":"Mise à jour de zephir-client",
            "zephir_client configure":"Envoi de configuration",
            "zephir_client reboot":"Redémarrage du serveur",
            "zephir_client service_restart":"Redémarrage du service",
            "zephir_client reconfigure":"Reconfiguration",
            "zephir_client update_key regen_certs":"Renouvellement des clés d'enregistrement et certificats ssl",
            "zephir_client update_key":"Renouvellement des clés d'enregistrement"}

UUCPError = 'UUCPError'
    

NUMBERS = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z','a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z']

# fonction de conversion 
def convert_num(chaine):
    result=0
    digits = range(len(chaine))
    # on parcourt la chaine en partant de la fin
    digits.reverse()
    for i in range(len(chaine)):
        # valeur du digit en cours
        digit = NUMBERS.index(chaine[digits[i]])
        # valeur * longueur_base^position
        result = result + digit * len(NUMBERS).__pow__(i)
    return result
 
class UUCP:
    """wrapper d'uucp pour permettre la gestion des files dans zephir"""
    
    def __init__(self,peers=None):
        """initialisation de l'objet"""
        self.peers_ori=peers
        self.pool={}
        self._scan_pool()

    def _scan_pool(self, peer=None):
        """crée la liste d'attente en mémoire"""
        # on crée la liste des commandes et transferts actuels
        # si pas de noms de machines spécifiées, on recherche dans tous les systèmes
        if peer is not None:
            peers = [peer]
        else:
            if self.peers_ori is None:
                peers = os.listdir(UUCP_DIR)
                for peer in IGNORE_LIST:
                    if peer in peers:
                        peers.remove(peer)
            else:
                peers = self.peers_ori
        # parcours de /var/spool/uucp    
        for peer in peers:
            # répertoire uucp de ce correspondant
            peer_dir = os.path.abspath(UUCP_DIR+os.sep+peer)
            # liste des actions en attente
            lst={}
            if os.path.isdir(peer_dir+"/C."):
                for fic in os.listdir(peer_dir+"/C."):
                    # on récupère la date de création de la commande (timestamp)
                    date_creat = os.stat(peer_dir+"/C./"+fic)[-1]
                    f=file(peer_dir+"/C./"+fic)
                    l=f.read().strip().split('\n')[0]
                    f.close()
                    # séparation des paramètres
                    # ex de lignes:
                    # S /var/lib/zephir/conf/0210056X/4/config-zephir.tar ~ root -Cd D.0008 0644
                    # S D.X0007 X.zephirN0007 root -C D.X0007 0666
                    args=l.split()[1]
                    script=l.split()[5]
                    seq_number=convert_num(script[-4:])
                    # si on a une commande
                    type_cmd = "transfert"
                    if args.startswith("D.X"):
                        type_cmd = "execute"
                        # on regarde laquelle dans le fichier correspondant
                        f=file(peer_dir+"/D.X/"+script)
                        l=f.read().strip().split('\n')
                        f.close()
                        for line in l:
                            line = line.strip()
                            if line[:2] == "C ":
                                args = line[2:]
                                # on regarde si c'est une commande connue
                                cmds = COMMANDS.keys()
                                cmds.sort(reverse=True)
                                for cmd in cmds:
                                    if line[2:].startswith(cmd):
                                        args = line[2:].replace(cmd, COMMANDS[cmd])
                                        break
                                
                    lst[seq_number]=(type_cmd,args,script,fic,date_creat)
                            
            self.pool[peer]=lst
                
                
    def add_cmd(self,peer,commande):
        """met en place une commande distante"""
        peer_dir = os.path.abspath(UUCP_DIR+os.sep+peer)
        # on regarde si on a le droit d'écrire
        timeout = 0
        while os.path.isfile(LOCK):
            # si non on attend la libération des ressources
            timeout += 1
            # si plus de 5 secondes d'attente, il y a surement un problème
            if timeout > 10:
                # print "Problème d'accès concurrent, fichier %s présent" % LOCK
                raise UUCPError, "Problème d'accès concurrent, fichier %s présent" % LOCK
            time.sleep(0.5)
        # on bloque les autres appels à cette fonction en cas de
        # demandes simultanées
        lock=file(LOCK,"w")
        lock.close()
        # on recherche le grade à affecter à la commande (pour assurer l'ordre d'execution)
        self._scan_pool(peer)
        used_grades = []
        if peer in self.pool:
            cmd_files = [uucmd[3] for uucmd in self.pool[peer].values()]
            for uucmd in cmd_files:
                cmd_grade = convert_num(uucmd[2])
                if cmd_grade not in used_grades:
                    used_grades.append(cmd_grade)
        if used_grades != []:
            new_grade = max(used_grades) + 1
        else:
            # grade de départ pour les commandes : 'O' (les transferts de fichiers ont 'N' par défaut)
            new_grade = 24
        # si on est déjà au grade maximum, on le conserve (ne devrait pas arriver sauf si file d'attente bloquée longtemps ?)
        if new_grade >= len(NUMBERS):
            new_grade = NUMBERS[-1]
        else:
            new_grade = NUMBERS[new_grade]
        # construction de l'appel à uux
        cmd = "%s -g %s -r '%s!%s >%s!%s'" % (CMD_UUX,new_grade,peer,commande,peer,LOG_FILE)
        try:
            # appel systeme
            res=os.system(cmd)
            # on ajoute la commande à la liste des commandes de ce peer
            if res == 0:
                # recherche du fichier correspondant (dernier fichier créé dans "C.")
                tempbuf=tempfile.mktemp()
                os.system("/bin/ls -t %s" % peer_dir+"/C./ > %s" % tempbuf)
                output=file(tempbuf)
                res2=output.read()
                output.close()
                os.unlink(tempbuf)
                # on récupère le nom du fichier le plus récent
                filename = res2.split()[0]
                # nom du script correspondant
                f=file(peer_dir+"/C./"+filename)
                l=f.read().strip().split('\n')[0]
                f.close()
                script = l.split()[5]
                # on déduit le n° dans la file à partir du nom de script
                seq_num=convert_num(script[-4:])
                if not self.pool.has_key(peer):
                    self.pool[peer]={}
                self.pool[peer][seq_num]=("execute",commande,script,filename)
                # on libère la ressource
                os.unlink(LOCK)
                return seq_num
            else:
                os.unlink(LOCK)
                raise Exception("uux2 a retourné une erreur")
        except Exception,e:
            os.unlink(LOCK)
            # print """erreur lors de la préparation de l'exécution de %s : %s""" % (commande,e)
            raise UUCPError, """erreur lors de la préparation de l'exécution de %s : %s""" % (commande,str(e))

    def add_file(self,peer,fichier,destination="~"):
        """prépare l'envoi d'un fichier"""
        peer_dir = os.path.abspath(UUCP_DIR+os.sep+peer)
        # construction de l'appel à uucp
        cmd = "%s -r %s %s\\!%s" % (CMD_UUCP,fichier,peer,destination)
        # on regarde si on a le droit d'écrire
        timeout = 0
        while os.path.isfile(LOCK):
            # si non on attend la libération des ressources
            timeout += 1
            # si plus de 5 secondes d'attente, il y a surement un problème
            if timeout > 10:
                # print "Problème d'accès concurrent, fichier %s présent" % LOCK
                raise UUCPError, "Problème d'accès concurrent, fichier %s présent" % LOCK
            time.sleep(0.5)
        # on bloque les autres appels à cette fonction en cas de
        # demandes simultanées
        lock=file(LOCK,"w")
        lock.close()
        try:
            # appel systeme
            res=os.system(cmd)
            # on ajoute le transfert à la liste des transferts de ce peer
            if res == 0:
                # recherche du fichier correspondant
                tempbuf=tempfile.mktemp()
                os.system("/bin/ls -t %s" % peer_dir+"/C./ > %s" % tempbuf)
                output=file(tempbuf)
                res2=output.read()
                output.close()
                os.unlink(tempbuf)
                # on récupère le nom du fichier le plus récent
                filename = res2.split()[0]
                # nom du script correspondant
                f=file(peer_dir+"/C./"+filename)
                l=f.read().strip().split('\n')[0]
                f.close()
                script = l.split()[5]
                # on déduit le n° dans la file à partir du nom de script
                seq_num=convert_num(script[-4:])
                if not self.pool.has_key(peer):
                    self.pool[peer]={}
                self.pool[peer][seq_num]=("transfert",fichier,script,filename)
                # on libère la ressource
                os.unlink(LOCK)
                return 0
            else:
                os.unlink(LOCK)
                raise UUCPError, """echec à l'exécution de uucp"""
        except Exception,e:
            os.unlink(LOCK)
            raise UUCPError, """erreur lors de la préparation du transfert de %s : %s""" % (fichier,e)

    def _create_liste(self,type_cmd,peers):
        """fonction interne qui liste les actions d'un type particulier"""
        cmds = {}
        # pour chaque peer
        for peer in peers:
            cmds[peer] = []
            if not self.pool.has_key(peer):
                continue
            # on parcourt la liste des actions en attente
            numeros = self.pool[peer].keys()
            numeros.sort()
            for n in numeros:
                action=self.pool[peer][n]
                # si c'est une commande distante
                if action[0] == type_cmd:
                    # on l'ajoute à la liste
                    cmds[peer].append((n,action[1]))
        return cmds

    def check_timeout(self,max_time,peer=None):
        """vérifie si il existe des commandes plus anciennes
        que max_time pour un serveur donné (ou tous)
        @param max_time: age maximum en seconde accepté pour une commande
        retourne un dictionnaire {serveur:liste des ids de commande trop anciens}"""
        dic_res = {}
        if peer is not None:
            peers=[peer]
        else:
            self._scan_pool()
            peers=self.pool.keys()
        for serveur in peers:
            timeouts = []
            if self.pool.has_key(serveur):
                for seq_num, data in self.pool[serveur].items():
                    test_time = time.localtime(float(data[-1] + max_time))
                    # si timeout (date de création + délai > date actuelle)
                    if test_time < time.localtime():
                        # on renvoie le n° de commande et sa date de création
                        timeouts.append((seq_num, time.ctime(data[-1])))
            if timeouts != []:
                dic_res[serveur] = timeouts
        return dic_res

            

    def list_cmd(self,peer=None):
        """renvoie la liste des commandes en attente"""
        if peer is not None:
            peers=[peer]
        else:
            self._scan_pool()
            peers=self.pool.keys()
        return self._create_liste("execute",peers)

    def list_files(self,peer=None):
        """renvoie la liste des transferts en attente"""
        if peer is not None:
            peers=[peer]
        else:
            self._scan_pool()
            peers=self.pool.keys()
        return self._create_liste("transfert",peers)

    def remove_cmd(self,peer,num_cmd):
        """supprime une commande ou un transfert"""
        type_cmd,fichier,script,filename,date_creat = self.pool[peer][int(num_cmd)]
        peer_dir = os.path.abspath(UUCP_DIR+os.sep+peer)
        # on regarde si on a le droit d'écrire
        timeout = 0
        while os.path.isfile(LOCK):
            # si non on attend la libération des ressources
            timeout += 1
            # si plus de 5 secondes d'attente, il y a surement un problème
            if timeout > 10:
                # print "Problème d'accès concurrent, fichier %s présent" % LOCK
                raise UUCPError, "Problème d'accès concurrent, fichier %s présent" % LOCK
            time.sleep(0.5)
        # on bloque les autres appels à cette fonction en cas de
        # demandes simultanées
        lock=file(LOCK,"w")
        lock.close()
        # supression du fichier correspondant dans C.
        try:
            os.unlink(peer_dir+'/C./'+filename)
            if type_cmd == "transfert":
                # supression de la copie du fichier dans D.
                os.unlink(peer_dir+'/D./'+script)
            else:
                # supression du script correspondant dans D.X
                os.unlink(peer_dir+'/D.X/'+script)
            # on libère la ressource
            os.unlink(LOCK)
        except:
            # print "erreur lors de la suppression des fichiers"
            os.unlink(LOCK)
            raise UUCPError, "erreur lors de la suppression des fichiers"
        
        del(self.pool[peer][int(num_cmd)])
        return 0
        

    def flush(self,peers=None):
        """supprime toute la file d'attente"""
        if peers is None:
            for i in os.listdir(UUCP_DIR):
                if i not in IGNORE_LIST:
                    try:
                        # on supprime le répertoire d'échange
                        if os.path.isdir(UUCP_DIR+os.sep+i):
                            shutil.rmtree(UUCP_DIR+os.sep+i)
                        self.pool[i]={}
                    except:
                        raise UUCPError, """erreur de suppression de la file d'attente de %s""" % i
        else:
            try:
                for peer in peers:
                    if os.path.isdir(UUCP_DIR+os.sep+peer):
                        shutil.rmtree(UUCP_DIR+os.sep+peer)
                    self.pool[peer]={}
            except:
                raise UUCPError, """erreur de supression de la file d'attente"""
        return 0

uucp_pool = UUCP()

if __name__ == '__main__':
    peers = ["0210056X-1","0210056X-2","0210056X-3","0210056X-4","0210056X-5"]
    uucp=UUCP(peers)
    peer = ""
    while not peer in peers:
        peer = raw_input("\nvoir la file d'attente de : ")
    print "\ncommandes :\n"
    for cmd in uucp.list_cmd(peer)[peer]:
        print "  "+str(cmd)

    print "\ntransferts :\n"
    for cmd in uucp.list_files(peer)[peer]:
        print "  "+str(cmd)
    print '\n'
