# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# xmlrpceole.py
#  
# Version modifiée du serveur XMLRPC twisted pour gérer
# l'authentification et la gestion des droits
#       
###########################################################################
import twisted
try:
    assert twisted.__version__[0] == '2'
    from twisted.web import http
except:
    from twisted.protocols import http
from twisted.web import error
from zephir.backend import config
from zephir.backend.lib_backend import ResourceAuthError

from twisted.internet import defer,protocol,reactor
from twisted.web import xmlrpc,server
from zephir.eolerpclib import xmlrpclib
import psycopg2 as PgSQL
import time

try:
    import ldap
except:
    pass
    

def convert(objet):
    """Transforme les objets unicode contenus dans un objet en chaines
    """
    if type(objet) == tuple:
        l = []
        for item in objet:
            l.append(convert(item))
        return l
    if type(objet) == list:
        l = []
        for item in objet:
            l.append(convert(item))
        return l
    if type(objet) == dict:
        dico={}
        for cle in objet.keys():
            dico[cle] = convert(objet[cle])
        return dico
    if type(objet) == unicode:
        string =  objet.encode(config.charset)
        return string
    return objet


# serveur sécurisé XMLPRC
class XMLRPCEole(xmlrpc.XMLRPC):
    
    
    def __init__(self):
        xmlrpc.XMLRPC.__init__(self)
        # définition des autorisations pour chaque utilisateur
        self.serveur_ldap = config.ADRESSE_LDAP
        # reload_perms : indique que les permissions ont changé
        # et qu'il faut les recharger
        self.reload_perms = 0
        self.load_groupes()

    def load_groupes(self):
        # chargement des groupes de droits
        cx = PgSQL.connect(database='zephir',user='zephir',password=config.DB_PASSWD)
        db_cursor = cx.cursor()
        db_cursor.execute("""select id,libelle,droits from groupes_droits""")
        res = db_cursor.fetchall()
        db_cursor.close()
        cx.close()
        self.groupes={}
        for groupe in res:
            self.groupes[int(groupe[0])]=[str(res[1]),eval(str(groupe[2]))]

    def render(self,request):
        """examine la requête transmise par le client et apelle la procédure
        correspondante si ses autorisations sont suffisantes"""
        # test de l'authentification
        cred_user = request.getUser()
        cred_password = request.getPassword()
        # dans le cas de l'utilisateur zephir, on ne fait pas de vérification sur ldap
        if cred_user != 'zephir' or cred_password != 'zephir':
            # on tente une connexion au serveur ldap avec ce login et mot de passe
            retry_auth = 0
            while retry_auth < 2:
                try:
                    if cred_password == "":
                        cred_password = None
                    query=ldap.open(self.serveur_ldap)
                    # on récupère le dn complet de l'utilisateur
                    result=query.search_s(config.BASE_LDAP, ldap.SCOPE_SUBTREE, "(uid="+cred_user+")")
                    cred_dn = result[0][0]
                    query.simple_bind_s(cred_dn,cred_password)
                    query.unbind()
                except ldap.SERVER_DOWN:
                    print "ldap server not responding, retrying authentification"
                    time.sleep(0.3)
                    retry_auth += 1
                except:
                    # erreur d'authentification
                    # on retourne un message d'erreur
                    print "\nauthentification incorrecte : ",request.host.host
                    errpage = error.ErrorPage(http.UNAUTHORIZED,"Unauthorized","401 Authentication required")
                    return errpage.render(request)
                    retry_auth = 2
                else:
                    retry_auth = 2
                    pass
                    # print "\nclient authentifié : ",request.getHost()[1]
            
        request.content.seek(0, 0)
        args, functionPath = xmlrpclib.loads(request.content.read())
        args_list=[]
        for arg in args:
            arg_conv=convert(arg)
            args_list.append(arg_conv)
        args = tuple(args_list)

        # on vérifie si l'utilisateur a le droit d'utiliser cette fonction
        # on récupère les groupes de droits de l'utilisateur
        cx = PgSQL.connect(database=config.DB_NAME,user=config.DB_USER,password=config.DB_PASSWD)
        cursor=cx.cursor()
        cursor.execute("""select droits from users where login='%s'""" % cred_user)
        rs = cursor.fetchone()
        cursor.close()
        cx.close()
        droits = []
        # on rassemble toutes les fonctions auxquelles on a droit
        if rs == [] or rs is None:
            groupe = []
        else:
            for groupe in eval(rs[0]):
                droits.extend(self.groupes[groupe][1])
        try:
            # on regarde si on a le droit d'executer la fonction
            if functionPath not in droits:
                # fonction interdite
                if request.host.host not in ["localhost", "127.0.0.1"]:
                    host_addr = " (%s)" % request.host.host
                else:
                    host_addr = ""
                print "\nutilisation de la fonction %s interdite pour %s%s" % (functionPath,cred_user,host_addr)
                errpage = error.ErrorPage(http.UNAUTHORIZED,"Unauthorized","erreur, ressource %s non autorisée !" % (request.uri))
                return errpage.render(request)
        except:
            print "\n pas d'autorisations pour " + cred_user + " !" 
            errpage = error.ErrorPage(http.UNAUTHORIZED,"Unauthorized","erreur, ressource %s non autorisée !" % (request.uri))
            return errpage.render(request)
        # fonction autorisée
        try:
            function = self._getFunction(functionPath)
        except xmlrpc.NoSuchFunction:
            self._cbRender(
                xmlrpclib.Fault(self.NOT_FOUND, "no such function %s" % functionPath),
                request
            )
        else:
            request.setHeader("content-type", "text/xml")
            defer.maybeDeferred(function, cred_user, *args).addErrback(
                self.ebRender, request
            ).addCallback(
                self._cbRender, request
            )
        return server.NOT_DONE_YET

    def ebRender(self, ex, request):
        """errback intermédiaire pour catcher les ressources non autorisées"""
        if ex.type == ResourceAuthError:
            errpage = error.ErrorPage(http.UNAUTHORIZED,"Unauthorized","erreur, autorisations insuffisantes (%s) !" % (str(ex.value)))
            print "\ntentative d'accès à une ressource interdite pour %s : %s" % (request.getUser(),str(ex.value))
            return errpage.render(request)
        else:
            self._ebRender(ex)

